@@ -1,26 +1,11 @@
-/* This file depends on libsupport.h having been included -- see hostenv.h */
+/* This file depends on  config.h  having been included -- see hostenv.h */
 
 #include <sys/types.h>
 
-
-#if	defined(USE_ALLOCA) || defined(HAVE_ALLOCA_H)
-	/* Actually compiler dependent.. */
-# if	!defined(ALLOCA_H) || defined(HAVE_ALLOCA_H)
-#  include <alloca.h>	/* This is the most common default location.. */
-# else
-#  include ALLOCA_H
-# endif
-#else /* No USE_ALLOCA */
-#if 0
-# if	defined(__GNUC__) || defined(__STDC__)
-#  define USE_ALLOCA
-#  define HAVE_ALLOCA_H
-#  include <alloca.h>
-# endif
+#include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <time.h>
 #endif
-#endif /* No USE_ALLOCA */
-
-#include TIME_H
 
 #include <stdio.h>
 
@@ -36,7 +21,7 @@
    extern char *realloc (); */
 #endif
 
-#ifdef USE_SYSPROTOS
+#ifdef USE_SYSPROTOS__xx
 
 extern void _exit();
 extern void abort();
