@@ -4,7 +4,7 @@
  */
 
 #include "hostenv.h"
-#ifdef	USE_INET
+#ifdef	HAVE_NETDB_H
 #include <netdb.h>
 
 /*
@@ -80,4 +80,4 @@ hp_addr_randomize(hp)
 	alist = oalist;
 }
 
-#endif	/* USE_INET */
+#endif	/* HAVE_RESOLVER */
