@@ -9,21 +9,18 @@
  *  abstracted away from the source code.
  */
 
+#ifdef HAVE_CONFIG_H /* In GNU autoconfig environment, most likely */
+#include "config.h"
+#endif
 #include <stdio.h>
 #include <errno.h>
 extern int errno;		/* Some errno.h don't declare this */
-#ifdef IN_ZMAILER
-#include "../include/libsupport.h"
 #ifdef STDC_HEADERS
 # define STDHEADERS
 #endif
-#ifdef USE_STDLIB
+#ifdef HAVE_UNISTD_H
 # define STDHEADERS
 #endif
-#ifdef USE_MMAP
-# define HAVE_MMAP
-#endif
-#endif
 
 /* 
  *  ANSI defines malloc to return void *, and take size_t as an
