@@ -15,9 +15,28 @@
 #include "interpret.h"
 #include "io.h"		/* redefines stdio routines */
 #include "shconfig.h"
-#ifdef	USE_UNIONWAIT
-#include <sys/wait.h>
-#endif	/* USE_UNIONWAIT */
+
+#ifdef  HAVE_WAITPID
+# include <sys/wait.h>
+#else
+# ifdef HAVE_WAIT3
+#  include <sys/wait.h> /* Has BSD wait3() */
+# else
+#  ifdef HAVE_SYS_WAIT_H /* POSIX.1 compatible */
+#   include <sys/wait.h>
+#  else /* Not POSIX.1 compatible, lets fake it.. */
+extern int wait();
+#  endif
+# endif
+#endif
+
+#ifndef WIFEXITED
+# define WIFEXITED(s) (((s) >> 8) & 0377)
+#endif
+#ifndef WIFSIGNALED
+# define WIFSIGNALED(s) ((s) & 0177)
+#endif
+
 
 #ifndef	HZ
 #define	HZ	60
@@ -1019,14 +1038,8 @@ sh_wait(argc, argv)
 	char *argv[];
 {
 	int retcode, pid;
-#ifdef USE_UNIONWAIT
-	union wait status;
-
-	status.w_termsig = 0;
-	status.w_retcode = 0;
-#else	/* !USE_UNIONWAIT */
 	int status = 0;
-#endif	/* !USE_UNIONWAIT */
+
 	if (argc > 2 || (argc == 2 && !isdigit(argv[1][0]))) {
 		fprintf(stderr, USAGE_WAIT, argv[0]);
 		return 1;
@@ -1037,25 +1050,14 @@ sh_wait(argc, argv)
 	}
 	if (pid < 0)		/* no more children */
 		return 0;
-#ifdef	USE_UNIONWAIT
-	if (status.w_termsig) {
-		fprintf(stderr, "%s", strsignal(status.w_termsig));
-		if (status.w_coredump)
-			fprintf(stderr, CORE_DUMPED);
-		fprintf(stderr, "\n");
-		retcode = 0200 + status.w_termsig;
-	} else
-		retcode = status.w_retcode;
-#else	/* !USE_UNIONWAIT */
-	if ((status&0177) > 0) {
-		fprintf(stderr, "%s", strsignal(status&0177));
+	if (WIFSIGNALED(status) > 0) {
+		fprintf(stderr, "%s", strsignal(WIFSIGNALED(status)));
 		if (status&0200)
 			fprintf(stderr, CORE_DUMPED);
 		fprintf(stderr, "\n");
-		retcode = 0200 + (status&0177);
+		retcode = 0200 + WIFSIGNALED(status);
 	} else
-		retcode = (status>>8)&0377;
-#endif	/* !USE_UNIONWAIT */
+		retcode = WIFEXITED(status);
 	return retcode;
 }
 
