@@ -27,11 +27,26 @@
 #include "io.h"
 #include "shconfig.h"
 
-#ifdef	USE_UNIONWAIT
-#include <sys/wait.h>
-#endif	/* USE_UNIONWAIT */
-#ifdef USE_WAITPID
-#include <sys/wait.h>
+
+#ifdef  HAVE_WAITPID
+# include <sys/wait.h>
+#else
+# ifdef HAVE_WAIT3
+#  include <sys/wait.h> /* Has BSD wait3() */
+# else
+#  ifdef HAVE_SYS_WAIT_H /* POSIX.1 compatible */
+#   include <sys/wait.h>
+#  else /* Not POSIX.1 compatible, lets fake it.. */
+extern int wait();
+#  endif
+# endif
+#endif
+
+#ifndef WIFEXITED
+# define WIFEXITED(s) (((s) >> 8) & 0377)
+#endif
+#ifndef WIFSIGNALED
+# define WIFSIGNALED(s) ((s) & 0177)
 #endif
 
 #ifndef	SIGCHLD
@@ -39,16 +54,16 @@
 #endif	/* SIGCHLD */
 
 extern FILE *runiofp;
-extern SIGNAL_TYPE (*orig_handler[])();
+extern RETSIGTYPE (*orig_handler[])();
 extern char *path_hash();
 extern char *progname;
 extern char *strerror();
 extern char *strsignal();
 extern char *strnsave();
-extern int addbuffer();
+extern int addbuffer __((u_char*, int, int, struct osCmd*));
 extern int errno;
 extern int interrupted;
-extern int runio();
+extern int runio __((struct IOop **));
 extern int sh_builtin();
 extern struct codedesc *interpret();
 extern struct conscell *v_find();
@@ -81,13 +96,12 @@ int smask = DEFAULT_OPEN_MASK;
 					} else if (P < 0)\
 						perror("fork");\
 					_exit(0);\
-				} else if (P < 0)
+				} else if (P > 0) { \
+					int sTaTuS; /* reap the child */ \
+					wait(&sTaTuS); \
+				} else
 
-#ifdef	USE_UNIONWAIT
-typedef union wait status_t;
-#else	/* !USE_UNIONWAIT */
 typedef int status_t;
-#endif	/* !USE_UNIONWAIT */
 
 int reapableTop = 0;			/* top of stack of repable processes */
 STATIC int reapable[MAXNPROC];		/* stack of reapable process ids */
@@ -165,14 +179,10 @@ execute(c, caller, oretcode, name)
 	int n;
 	register struct conscell *sl, *l;
 	struct conscell *tmp;
-	SIGNAL_TYPE (*oquit_handler)(), (*oint_handler)(), (*oterm_handler)();
-	SIGNAL_TYPE (*ochld_handler)();
+	RETSIGTYPE (*oquit_handler)(), (*oint_handler)(), (*oterm_handler)();
+	RETSIGTYPE (*ochld_handler)();
 
-#ifdef	USE_UNIONWAIT
-	status.w_status = 0;
-#else	/* !USE_UNIONWAIT */
 	status = 0;
-#endif	/* !USE_UNIONWAIT */
 
 	ochld_handler = SIG_DFL;
 	globalcaller = caller;
@@ -565,29 +575,16 @@ execute(c, caller, oretcode, name)
 			trapexit(retcode);
 		return retcode;
 	}
-#ifdef	USE_UNIONWAIT
-	if (status.w_termsig) {
-		if (status.w_termsig != SIGINT) {
-			fprintf(stderr, "%s", strsignal(status.w_termsig));
-			if (status.w_coredump)
-				fprintf(stderr, CORE_DUMPED);
-		}
-		fprintf(stderr, "\n");
-		retcode = 0200 + status.w_termsig;
-	} else
-		retcode = status.w_retcode;
-#else	/* !USE_UNIONWAIT */
-	if ((status&0177) > 0) {
-		if ((status&0177) != SIGINT) {
-			fprintf(stderr, "%s", strsignal(status&0177));
+	if (WIFSIGNALED(status) > 0) {
+		if (WIFSIGNALED(status) != SIGINT) {
+			fprintf(stderr, "%s", strsignal(WIFSIGNALED(status)));
 			if (status&0200)
 				fprintf(stderr, CORE_DUMPED);
 		}
 		fprintf(stderr, "\n");
-		retcode = 0200 + (status&0177);
+		retcode = 0200 + WIFSIGNALED(status);
 	} else
-		retcode = (status>>8)&0377;
-#endif	/* USE_UNIONWAIT */
+		retcode = WIFEXITED(status);
 	if (isset('t'))
 		trapexit(retcode);
 	return retcode;
@@ -606,8 +603,8 @@ readstring(fd, command)
 	u_char buf[BUFSIZ];	/* read this size chunk at a time */
 
 	state = 0;
-	while ((n = read(fd, buf, sizeof buf)) > 0)
-		state = addbuffer(buf, n, state, command);
+	while ((n = read(fd, &buf[0], sizeof(buf))) > 0)
+		state = addbuffer(&buf[0], n, state, command);
 }
 
 int
@@ -621,16 +618,17 @@ addbuffer(buf, len, state, command)
 
 	ncp = cp = buf;
 	for (cp = buf; --len >= 0; ++cp) {
-		if (!isascii(*cp))
-			*ncp++ = *cp, state = 1;
+		u_char c = *cp;
+		if (!isascii(c))
+			*ncp++ = c, state = 1;
 		else if (state) {	/* break on whitespace */
-			if (isspace(*cp)) {
-				*ncp++ = (*cp == '\n') ? *cp : ' ';
+			if (isspace(c)) {
+				*ncp++ = (c == '\n') ? c : ' ';
 				state = 0;
 			} else
-				*ncp++ = *cp;
-		} else if (!isspace(*cp)) /* ignoring whitespace */
-			*ncp++ = *cp, state = 1;
+				*ncp++ = c;
+		} else if (!isspace(c)) /* ignoring whitespace */
+			*ncp++ = c, state = 1;
 	}
 	/* wrap up the stuff we got so far into a string buffer */
 	if (ncp > buf) {
@@ -870,13 +868,13 @@ runio(ioopp)
 	 * interpreting the list.
 	 */
 	ioop = *ioopp;
-	for (ioprev = 0; ioop != 0; ioop = ionext) {
+	for (ioprev = NULL; ioop != NULL; ioop = ionext) {
 		ionext = ioop->next;
 		ioop->next = ioprev;
 		ioprev = ioop;
 	}
 	errflag = 0;
-	for (ioop = ioprev, ionext = NULL; ioop != 0; ioop = ioop->next) {
+	for (ioop = ioprev, ionext = NULL; ioop != NULL; ioop = ioop->next) {
 		switch (ioop->cmd) {
 #ifdef	S_IFIFO
 		case sIOopenPortal:	/* open named pipe */
