@@ -24,9 +24,28 @@
 #include "interpret.h"
 #include "io.h"
 #include "shconfig.h"
-#ifdef	USE_UNIONWAIT
-#include <sys/wait.h>
-#endif	/* USE_UNIONWAIT */
+
+
+#ifdef  HAVE_WAITPID
+# include <sys/wait.h>
+#else
+# ifdef HAVE_WAIT3
+#  include <sys/wait.h> /* Has BSD wait3() */
+# else
+#  ifdef HAVE_SYS_WAIT_H /* POSIX.1 compatible */
+#   include <sys/wait.h>
+#  else /* Not POSIX.1 compatible, lets fake it.. */
+extern int wait();
+#  endif
+# endif
+#endif
+
+#ifndef WIFEXITED
+# define WIFEXITED(s) (((s) >> 8) & 0377)
+#endif
+#ifndef WIFSIGNALED
+# define WIFSIGNALED(s) ((s) & 0177)
+#endif
 
 int magic_number = 3;		/* check id for precompiled script files */
 long bin_magic   = 1;		/* Another check-id -- exec file  st_ctime ? */
@@ -1735,7 +1754,6 @@ XXX: HERE! Must copy the output to PREVIOUS memory level, then discard
 			if (command->memlevel && command->rval == NULL
 			    && command->buffer == NULL
 			    && command->iocmd != ioIntoBuffer) {
-setlevel(MEM_SHCMD, command->memlevel); /* void routines clean their memory */
 				if (command->pgrp > 0)
 					jc_report(command->pgrp);
 			}
@@ -1917,13 +1935,8 @@ setlevel(MEM_SHCMD, command->memlevel); /* void routines clean their memory */
 				--ibt;
 			} else if (childpid > 0) {
 				/* in parent */
-#ifdef	USE_UNIONWAIT
-				while (wait((union wait *)0) != childpid)
+				while (wait(NULL) != childpid)
 					continue;
-#else	/* !USE_UNIONWAIT */
-				while (wait((int *)0) != childpid)
-					continue;
-#endif	/* USE_UNIONWAIT */
 				pc = code + argi1 - 1;
 			} else if (childpid < 0) {
 				/* error */
