@@ -9,8 +9,8 @@
  * if stdio allowed you to open strings.  Sigh.
  */
 
-#include <stdio.h>
 #include "hostenv.h"
+#include <stdio.h>
 #include <varargs.h>
 #include <fcntl.h>
 #include <sys/file.h>
@@ -131,7 +131,7 @@ siofputs(s, fp)
 	return 0;
 }
 
-#ifndef	USE_VFPRINTF
+#ifndef	HAVE_VPRINTF
 /* This is done in desperation... */
 int
 vfprintf(fp, fmt, ap)
@@ -154,7 +154,7 @@ vfprintf(fp, fmt, ap)
         count = _doprnt(fmt, ap, fp);
         return(ferror(fp)? EOF: count);
 }
-#endif	/* !USE_VFPRINTF */
+#endif	/* !HAVE_VPRINTF */
 
 int
 siofprintf(va_alist)
