@@ -73,41 +73,44 @@ struct db_kind {
 		  add_header, remove_header, print_header, count_header, owner_header, NULL, Nul } },
 { "unordered",	{ NULL, NULL, 0, 10, 0, NULL, search_seq, close_seq,
 		  add_seq, NULL, print_seq, count_seq, owner_seq, modp_seq, Nul } },
-#ifndef	USE_MMAP
+#ifndef	HAVE_MMAP
 { "ordered",	{ NULL, NULL, 0, 10, 0, NULL, search_bin, close_seq,
 		  NULL, NULL, print_seq, count_seq, owner_seq, modp_seq, Nul }	},
-#else /* USE_MMAP */ /* When using MMAP(), no cache is needed for ordered.. */
+#else /* HAVE_MMAP */ /* When using MMAP(), no cache is needed for ordered.. */
 { "ordered",	{ NULL, NULL, 0, 0, 0, NULL, search_bin, close_seq,
 		  NULL, NULL, print_seq, count_seq, owner_seq, modp_seq, Nul }	},
 #endif
 
-#ifdef	USE_HOSTS
+#ifdef	HAVE_HOSTS
 { "hostsfile",	{ HOSTS_FILE, NULL, 0, 10, 0, NULL, search_hosts, NULL,
 		  NULL, NULL, print_hosts, NULL, NULL, NULL, Nul } },
-#endif	/* USE_HOSTS */
-#ifdef	USE_RESOLV
+#endif	/* HAVE_HOSTS */
+#ifdef	HAVE_RESOLVER
+#ifndef RESOLV_CONF
+# define RESOLV_CONF "/etc/resolv.conf"
+#endif
 { "bind",	{ RESOLV_CONF, NULL, 0, 10, 0, NULL,
 		  search_res, NULL, NULL, NULL, NULL, NULL, NULL, NULL, Nul }	},
-#endif	/* USE_RESOLV */
-#ifdef	USE_NDBM
+#endif	/* HAVE_RESOLV */
+#ifdef	HAVE_NDBM_H
 { "ndbm",	{ NULL, NULL, 0, 10, 0, NULL, search_ndbm, close_ndbm,
 		  add_ndbm, remove_ndbm, print_ndbm, count_ndbm, owner_ndbm,
 		  modp_ndbm, Nul } },
-#endif	/* USE_NDBM */
-#ifdef	USE_GDBM
+#endif	/* HAVE_NDBM */
+#ifdef	HAVE_GDBM_H
 { "gdbm",	{ NULL, NULL, 0, 10, 0, NULL, search_gdbm, close_gdbm,
 		  add_gdbm, remove_gdbm, print_gdbm, count_gdbm, owner_gdbm,
 		  modp_gdbm, Nul } },
-#endif	/* USE_GDBM */
-#ifdef	USE_DBM
+#endif	/* HAVE_GDBM */
+#ifdef	HAVE_DBM
 { "dbm",	{ NULL, NULL, 0, 10, 0, NULL, search_dbm, close_dbm,
 		  add_dbm, remove_dbm, print_dbm, count_dbm, owner_dbm,
 		  NULL, Nul } },
-#endif	/* USE_DBM */
-#ifdef	USE_YP
+#endif	/* HAVE_DBM */
+#ifdef	HAVE_YP
 { "yp",		{ NULL, NULL, 0, 10, 0, NULL, search_yp, NULL,
 		  NULL, NULL, print_yp, NULL, owner_yp, NULL, Nul }	},
-#endif	/* USE_YP */
+#endif	/* HAVE_YP */
 { 0 }	/* proto_config is initialized from this entry */
 };
 
