@@ -6,7 +6,7 @@
 /* LINTLIBRARY */
 
 #include "mailer.h"
-#ifdef	USE_RESOLV
+#ifdef	HAVE_RESOLVER
 
 #undef	RFC974		/* MX/WKS/A processing according to RFC974 */
 
@@ -751,4 +751,4 @@ getrrtype(host, rrtype, ttlp, depth)
 	return NULL;
 }
 #endif	/* TRY_AGAIN */
-#endif	/* USE_RESOLV */
+#endif	/* HAVE_RESOLVER */
