@@ -56,9 +56,8 @@ struct headerinfo optional_hdrs[] = {
 };
 
 struct headerinfo envelope_hdrs[] = {
-#ifdef	USE_AUTH
-{ "authinfo",	nilHeaderSemantics,	nilUserType,	eAuthinfo	},
-#endif
+{ "authinfo",	nilHeaderSemantics,	nilUserType,	eIdentinfo	},
+{ "identinfo",	nilHeaderSemantics,	nilUserType,	eIdentinfo	},
 { "channel",		AnyWord,	nilUserType,	eChannel	},
 { "envid",	nilHeaderSemantics,	nilUserType,	eEnvid		},
 { "external",	nilHeaderSemantics,	nilUserType,	eExternal	},
@@ -192,7 +191,7 @@ find_header(db, name)
 	char *name;
 {
 	struct spblk *spl;
-	register char *cp, *bp, *nbuffer;
+	register char *cp, *bp;
 #ifdef	USE_ALLOCA
 	int len = strlen(name);
 	char *buffer = alloca(len+1);
@@ -214,6 +213,7 @@ find_header(db, name)
 	}
 	for (cp = name, bp = buffer; *cp != '\0'; ++cp, ++bp) {
 		if (bp - buffer > len - 2) {
+			register char *nbuffer;
 			len *= 2;
 			nbuffer = realloc(buffer, len);
 			bp = nbuffer + (bp - buffer);
