@@ -13,9 +13,9 @@
 #include <sys/file.h>
 #include <errno.h>
 #include "mail.h"
-#ifdef	USE_SYSLOG
+#ifdef	HAVE_SYSLOG
 #include <syslog.h>
-#endif	/* USE_SYSLOG */
+#endif	/* HAVE_SYSLOG */
 #include <unistd.h>
 
 #include "prototypes.h"
@@ -27,7 +27,7 @@
 #define _IOLBF  0200
 #endif  /* !_IOFBF */
 
-#ifdef	USE_LOCKF
+#ifdef	HAVE_LOCKF
 #ifdef	F_OK
 #undef	F_OK
 #endif	/* F_OK */
@@ -41,7 +41,7 @@
 #undef	R_OK
 #endif	/* R_OK */
 
-#endif	/* USE_LOCKF */
+#endif	/* HAVE_LOCKF */
 
 extern int hdr_status();
 extern int efstat();
@@ -193,17 +193,17 @@ makeLetter(e, octothorp)
 	e->e_msgOffset = 0;
 	e->e_nowtime = now = time(NULL);
 	if (efstat(FILENO(e->e_fp), &(e->e_statbuf)) < 0) {
-#ifdef	USE_STATBLKSIZE
+#ifdef	HAVE_ST_BLKSIZE
 		e->e_statbuf.st_blksize = 0;
-#endif	/* !USE_STATBLKSIZE */
+#endif	/* !HAVE_ST_BLKSIZE */
 		e->e_statbuf.st_mtime = e->e_nowtime;
 	}
 	e->e_localtime = *(localtime(&(e->e_statbuf.st_mtime)));
-#ifdef	USE_STATBLKSIZE
+#ifdef	HAVE_ST_BLKSIZE
 	initline(e->e_statbuf.st_blksize);
-#else	/* !USE_STATBLKSIZE */
+#else	/* !HAVE_ST_BLKSIZE */
 	initline(4096);
-#endif	/* USE_STATBLKSIZE */
+#endif	/* !HAVE_ST_BLKSIZE */
 
 	inheader = 0;
 	while ((n = getline(e->e_fp)) > !octothorp) {
@@ -214,6 +214,7 @@ makeLetter(e, octothorp)
 		    linebuf[n-1] == '\n') {
 			--n;
 			linebuf[n-1] = '\n';
+			if (n <= !octothorp) break;
 		}
 		/* Ok, now we can proceed with the original agenda.. */
 		i = hdr_status(linebuf, linebuf, n);
@@ -477,7 +478,9 @@ squirrel(e, text)
 
 	if (elink(e->e_file, path) < 0) {
 		sprintf(path, "..%d", e->e_statbuf.st_ino);
-		elink(e->e_file, path);
+		erename(e->e_file, path);
+	} else {
+		eunlink(e->e_file);
 	}
 	fprintf(stderr, "squirrel: %d saved for inspection: %s\n",
 			e->e_statbuf.st_ino, text);
@@ -1072,6 +1075,8 @@ sequencer(e, file)
 				     "%s: looping message, squirreled away!\n",
 					progname);
 				perr = PERR_LOOP;
+				squirrel(e,"looping message, exceeded maxReceived");
+				/* XXX: Should do "loop-count-exceeded"! */
 				break;
 			}
 		}
@@ -1100,7 +1105,10 @@ sequencer(e, file)
 		 * the message when the problem is fixed, so we don't tell
 		 * the originator that anything went wrong.
 		 */
-		perr = PERR_ENVELOPE;
+		defer(e);
+		/* Often the defer() becomes resubmited via cron, but lets
+		   hope the postmaster will see things manually.. */
+		return PERR_ENVELOPE;
 	}
 	e->e_from_trusted = makequad();
 	e->e_from_resolved = makequad();
@@ -1127,11 +1135,6 @@ sequencer(e, file)
 
 	/* put pertinent message information in the zsh environment */
 	setenvinfo(e);
-	if (perr) {
-		/* XX: report this ??? */
-		eunlink(file);
-		return perr;
-	}
 
 	FindEnvelope(eEnvid);
 	envid = NULL;
@@ -1745,32 +1748,16 @@ sequencer(e, file)
 		 * stuff.  The only safe way is to open it with the permissions
 		 * of the owner of the message file.
 		 */
-#ifdef	USE_SETREUID
 		setreuid(0, e->e_statbuf.st_uid);
-#else	/* SVID */
-#ifdef	USE_SETEUID
-		seteuid(e->e_statbuf.st_uid);
-#else
-		/* Only reversible under recent SVID! */
-		setuid(e->e_statbuf.st_uid);
-#endif	/* !USE_SETEUID */
-#endif	/* !USE_SETREUID */
+
 		if ((vfp = fopen(verbosefile, "a")) != NULL) {
-			fseek(vfp, (long)0, 2);
-			setvbuf(vfp, (char *)NULL, _IOLBF, 0);
+			fseek(vfp, 0L, 2);
+			setvbuf(vfp, NULL, _IOLBF, 0);
 			fprintf(vfp, "router processed message %s\n", file);
 			fprintf(ofp, "%c%c%s\n", _CF_VERBOSE, _CFTAG_NORMAL,
 				verbosefile);
 		}
-#ifdef	USE_SETREUID
 		setreuid(0, 0);
-#else	/* SVID */
-#ifdef	USE_SETEUID
-		seteuid(0);
-#else
-		setuid(0);	/* Only works under recent SVID! */
-#endif	/* !USE_SETEUID */
-#endif	/* !USE_SETREUID */
 	} else
 		vfp = NULL;
 
