@@ -294,8 +294,7 @@ mkMessageId(e, unixtime)
 	h->h_descriptor = msgidDesc;
 	h->h_stamp = newHeader;
 	h->h_next = 0;
-	ts = localtime(&unixtime);
-	cp = rfc822tz(ts, 1);
+	cp = rfc822tz(&unixtime, &ts, 1);
 	for (s = cp, p = tzbuf; *s != '\0'; ++s) {
 		if (isascii(*s) && isupper(*s))
 			*p++ = tolower(*s);
