@@ -12,6 +12,8 @@
 #include "scheduler.h"
 #include "prototypes.h"
 
+extern char *strchr();
+
 time_t qipcretry = 0;
 
 int	/* return non-zero when there are some childs
@@ -62,3 +64,38 @@ doagenda()
 	}
 	return (didsomething);
 }
+
+/* Do immediate scheduling of given host-indentifier. */
+void
+turnme(turnarg)
+char *turnarg;
+{
+	struct spblk *spl;
+	struct web *wp;
+	char *cp = strchr(turnarg,' ');
+	struct thread *ncuritem, *nncuritem;
+
+	/* caller has done 'strlower()' to our input.. */
+	if (cp) *cp++ = 0;
+	/* If cp is non-null, some additional args were present.. */
+
+	spl = sp_lookup(symbol(turnarg), spt_mesh[L_HOST]);
+	if (spl == NULL || (wp = (struct web *)spl->data) == NULL) {
+	  /* Not found, nothing to do.. */
+	  return;
+	}
+	
+	ncuritem = thread_head;
+	while (ncuritem != NULL) {
+	  /* Object pointed by  ncuritem  may disappear due to
+	     expiration at thread_start()	 */
+	  nncuritem = ncuritem->nexttr;
+
+	  if (wp == ncuritem->whost && ncuritem->proc == NULL) {
+	    ncuritem->wakeup = 0; /* Force its starttime! */
+	    thread_start(ncuritem);
+	    /* We MAY get multiple matches, though it is unlikely.. */
+	  }
+	  ncuritem = nncuritem;
+	}
+}   
