@@ -14,7 +14,7 @@
 #include "malloc.h"
 #include "splay.h"
 
-#ifndef	USE_SYSLOG
+#ifndef	HAVE_SYSLOG
 #define	syslog	(void)
 #endif
 
