@@ -1,7 +1,7 @@
 /*
  *	ZMailer 2.99.16+ Scheduler "threads" routines
  *
- *	Copyright Matti Aarnio <mea@nic.funet.fi> 1995
+ *	Copyright Matti Aarnio <mea@nic.funet.fi> 1995-96
  *
  *
  *	These "threads" are for book-keeping of information
@@ -12,7 +12,7 @@
 #include "prototypes.h"
 #include <ctype.h>
 #include <unistd.h>
-#include <stdlib.h>
+/* #include <stdlib.h> */
 
 /*
    Each vertex arrives into SOME thread, each of which belong to
@@ -373,9 +373,8 @@ if (verbose) printf("thread_linkin([%s/%s],%s/%d/%s,%d)\n",wc->name,wh->name,
 	/* char const *vp_chan = wc->name; */
 	/* char const *vp_host = wh->name; */
 
-	if (thrg_root == NULL) {
-	  (void) create_threadgroup(cep, wc, wh, cep->flags & CFG_WITHHOST, ce_fillin);
-	}
+	if (thrg_root == NULL)
+	  create_threadgroup(cep,wc,wh,cep->flags & CFG_WITHHOST,ce_fillin);
 
 	/*
 	 *  Search for matching config-entry, AND matching channel,
@@ -679,6 +678,9 @@ struct thread *thr;
 	struct vertex      *vp  = thr->vertices;
 	struct threadgroup *thg = thr->thgrp;
 	struct config_entry *ce = &(thr->thgrp->ce);
+	struct web         *ho  = vp->orig[L_HOST];
+	struct web         *ch  = vp->orig[L_CHANNEL];
+
 
 	if (syncstart) return 0;
 
@@ -696,18 +698,36 @@ struct thread *thr;
 
       re_pick:
 	if (thg->idleproc != NULL) {
-	  struct procinfo *proc;
+	  struct procinfo *proc, **ipp;
+
+	  /* Idle processor(s) exists, try to optimize by finding
+	     an idle proc with matching channel & host from previous
+	     activity. If not found, pick any. */
+	  
+	  ipp = &(thg->idleproc);
+	  /* There is at least one.. */
+	  proc  = *ipp;
+
+	  while (proc) {
+	    if (proc->ho == ho && proc->ch == ch)
+	      break;
+
+	    ipp  = &(proc->next);
+	    proc = *ipp;
+	  }
+	  if (proc == NULL) {
+	    /* None of the previous ones matched, pick the first anyway */
+	    ipp = &(thg->idleproc);
+	    /* There is at least one.. */
+	    proc  = *ipp;
+	  }
+	  /* Selected one of them.. */
+	  thr->proc     = proc;
+	  *ipp          = proc->next;
+	  proc->next    = NULL;
 
-	  /* Idle processor(s) exists, take the first of them */
-	  /* XXX: try to optimize by finding idle proc with matching
-	          channel & host from previous activity. If not found,
-		  pick any. */
-	  thr->proc     = thg->idleproc;
-	  proc          = thr->proc;
-	  thg->idleproc = proc->next;
 	  thg->idlecnt -= 1;
 	  --idleprocs;
-	  proc->next    = NULL;
 	  
 	  /* It may be that while we idled it, it died at the idle queue.. */
 	  if (proc->pid <= 0 || proc->tofd < 0)
@@ -717,9 +737,9 @@ struct thread *thr;
 	  proc->vertex  = thr->vertices;
 	  /* Thread-groups are made such that here at thread_start() we
 	     can always switch over in between threads */
-	  proc->ch = thr->vertices->orig[L_CHANNEL];
+	  proc->ch = ch;
 
-	  if (proc->ho != NULL) {
+	  if (proc->ho != NULL && proc->ho != ho) {
 	    /* Get rid of the old host web */
 	    proc->ho->kids -= 1;
 	    if (proc->ho->kids == 0 && proc->ho->link == NULL)
@@ -727,8 +747,11 @@ struct thread *thr;
 	  }
 
 	  /* Move the kid to this host web */
-	  proc->ho = thr->vertices->orig[L_HOST];
-	  proc->ho->kids += 1;
+	    
+	  if (proc->ho != ho) {
+	    proc->ho = ho;
+	    proc->ho->kids += 1;
+	  }
 
 	  /* Clean vertices 'proc'-pointers,  randomize the
 	     order of thread vertices  (or sort by spool file
