@@ -8,18 +8,17 @@
  */
 
 
+#include "hostenv.h"
 #include <stdio.h>
 #include <sys/param.h>
-#include "hostenv.h"
 #include "scheduler.h"
 #include "mail.h"
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <errno.h>
 #include <sys/file.h>
-#include NDIR_H
 #include "zmsignal.h"
-#include <stdlib.h>
+/* #include <stdlib.h> */
 #include <unistd.h>
 
 #include "prototypes.h"
@@ -33,11 +32,26 @@ static void reclaim __((int, int));
 static void waitandclose __((int));
 static void readfrom __((int));
 
-#ifdef	USE_UNIONWAIT
-#include <sys/wait.h>
+
+#ifdef  HAVE_WAITPID
+# include <sys/wait.h>
+#else
+# ifdef HAVE_WAIT3
+#  include <sys/wait.h> /* Has BSD wait3() */
+# else
+#  ifdef HAVE_SYS_WAIT_H /* POSIX.1 compatible */
+#   include <sys/wait.h>
+#  else /* Not POSIX.1 compatible, lets fake it.. */
+extern int wait();
+#  endif
+# endif
 #endif
-#ifdef	USE_WAITPID
-#include <sys/wait.h>
+
+#ifndef WIFEXITED
+# define WIFEXITED(s) (((s) >> 8) & 0377)
+#endif
+#ifndef WIFSIGNALED
+# define WIFSIGNALED(s) ((s) & 0177)
 #endif
 
 struct procinfo *cpids = NULL;
@@ -89,7 +103,19 @@ flush_child(proc)
 struct procinfo *proc;
 {
 	int rc, len;
-	if (proc->cmdlen != 0) {
+
+	if (proc->tofd < 0) {
+	  if (proc->vertex)
+	    proc->vertex->proc = NULL;
+	  if (proc->thread) {
+	    proc->thread->proc = NULL;
+	    proc->thread       = NULL;
+	  }
+	  proc->cmdlen = 0;
+	  return 0;
+	}
+
+	if (proc->cmdlen != 0 && proc->tofd >= 0) {
 	  /* We have some leftovers from previous feed..
 	     .. feed them now.  */
 	  rc = write(proc->tofd, proc->cmdbuf, proc->cmdlen);
@@ -99,9 +125,12 @@ struct procinfo *proc;
 	      /* Some real failure :-( */
 	      pipes_shutdown_child(proc->tofd);
 	      proc->tofd = -1;
-	      proc->vertex->proc = NULL;
-	      proc->thread->proc = NULL;
-	      proc->thread       = NULL;
+	      if (proc->vertex)
+		proc->vertex->proc = NULL;
+	      if (proc->thread) {
+		proc->thread->proc = NULL;
+		proc->thread       = NULL;
+	      }
 	      proc->cmdlen = 0;
 	      return -1;
 	    }
@@ -381,6 +410,9 @@ static void stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
 	  fd_nonblockingmode(tofd);
 }
 
+/* 
+ *  Reclaim the process slot -- this process is dead now.
+ */
 static void reclaim(fromfd, tofd)
 	int fromfd, tofd;
 {
@@ -486,7 +518,12 @@ static void waitandclose(fd)
 	reclaim(fd, cpids[fd].tofd);
 }
 
-#ifdef	USE_SELECT
+#ifdef	HAVE_SELECT
+
+#ifdef _AIX /* The select.h  defines NFDBITS, etc.. */
+# include <sys/types.h>
+# include <sys/select.h>
+#endif
 
 
 #if	defined(BSD4_3) || defined(sun)
@@ -516,16 +553,6 @@ typedef long	fd_mask;
 typedef	struct fd_set { fd_mask	fds_bits[1]; } fd_set;
 #endif	/* notdef */
 
-#ifdef _AIX
-#include <sys/types.h>
-#include <sys/select.h>
-/*	struct {
-		fd_set  fdsmask;
-		fd_set  msgsmask;
-	} readfds, writefds;
-*/
-#endif
-
 #ifndef	FD_SET
 #define	FD_SET(n, p)	((p)->fds_bits[0] |= (1 << (n)))
 #define	FD_CLR(n, p)	((p)->fds_bits[0] &= ~(1 << (n)))
@@ -534,12 +561,6 @@ typedef	struct fd_set { fd_mask	fds_bits[1]; } fd_set;
 #endif	/* !FD_SET */
 #endif	/* !NFDBITS */
 
-#if defined(_AIX) && !defined(FD_SET) /* Pre AIX 4.1 ? */
-#define _FD_SET(sock,var) FD_SET(sock,&var.fdsmask)
-#define _FD_CLR(sock,var) FD_CLR(sock,&var.fdsmask)
-#define _FD_ZERO(var) FD_ZERO(&var.fdsmask)
-#define _FD_ISSET(i,var) FD_ISSET(i,&var.fdsmask)
-#else
 #ifdef FD_SET
 #define _FD_SET(sock,var) FD_SET(sock,&var)
 #define _FD_CLR(sock,var) FD_CLR(sock,&var)
@@ -551,7 +572,6 @@ typedef	struct fd_set { fd_mask	fds_bits[1]; } fd_set;
 #define _FD_ZERO(var) var = 0
 #define _FD_ISSET(i,var) ((var & (1 << i)) != 0)
 #endif
-#endif
 
 int in_select = 0;
 
@@ -560,13 +580,8 @@ mux(timeout)
 time_t timeout;
 {
 	int	i, n, rc, maxf;
-#ifdef _AIX
-	struct { fd_set fdsmask; fd_set msgsmask; } rdmask;
-	struct { fd_set fdsmask; fd_set msgsmask; } wrmask;
-#else
 	fd_set	rdmask;
 	fd_set	wrmask;
-#endif
 	struct timeval tv;
 
 	if (in_select) fprintf(stderr,"**** recursed into mux()! ***\n");
@@ -685,11 +700,7 @@ queryipccheck()
 {
 	if (querysocket >= 0) {
 	  int	n;
-#ifdef _AIX
-	  struct { fd_set fdsmask; fd_set msgsmask; } mask;
-#else
 	  fd_set	mask;
-#endif
 	  struct timeval tv;
 
 	  tv.tv_sec = 0;
@@ -863,10 +874,8 @@ static void readfrom(fd)
 	      pcp = buf;	/* may be used below */
 	    }
 	  }
-#ifndef	USE_BSDNODELAY
 	  if (!dontbreak)
 	    break;
-#endif				/* !USE_BSDNODELAY */
 	}
 
 	if (verbose) {
@@ -972,41 +981,29 @@ system(name)
 /*
  *	Catch each child-process death, and reap them..
  */
-SIGNAL_TYPE sig_chld(signum)
+RETSIGTYPE sig_chld(signum)
 int signum;
 {
 	int pid;
 	int ok = 0;
 	int i;
-#ifdef USE_UNIONWAIT
-	union wait statloc;
-#else
 	int statloc;
-#endif
 
 	for (;;) {
 
-#ifdef USE_UNIONWAIT
-	  pid = wait3(&statloc,WNOHANG,NULL);
-#else
-#ifdef	USE_WAITPID
+#ifdef	HAVE_WAITPID
 	  pid = waitpid(-1,&statloc,WNOHANG);
+#else
+#ifdef  HAVE_WAIT3
+	  pid = wait3(&statloc,WNOHANG,NULL);
 #else
 	  pid = wait(&statloc);
 #endif
 #endif
 	  if (pid <= 0) break;
 
-#ifdef	WIFEXITED
 	  if (WIFEXITED(statloc)) ok = 1;
-#else
-      ERROR_ERROR_ERROR:
-#endif
-#ifdef WIFSIGNALED
-	if (WIFSIGNALED(statloc)) ok = 1;
-#else
-      ERROR_ERROR_ERROR:
-#endif
+	  if (WIFSIGNALED(statloc)) ok = 1;
 
 	  if (verbose)
 	    fprintf(stderr,"sig_chld() pid=%d, ok=%d, stat=0x%x\n",
