@@ -16,7 +16,6 @@
 #include "mail.h"
 #include "zmsignal.h"
 #include "ta.h"
-#include NDIR_H
 
 
 #define	PROGNAME	"errormail"
@@ -62,7 +61,7 @@ main(argc, argv)
 	char *channel, msgfilename[MAXPATHLEN+1];
 	int errflg, c;
 	struct ctldesc *dp;
-	SIGNAL_TYPE (*oldsig)();
+	RETSIGTYPE (*oldsig)();
 
 	SIGNAL_HANDLESAVE(SIGINT, SIG_IGN, oldsig);
 	if (oldsig != SIG_IGN)
@@ -169,10 +168,10 @@ FILE *mfp;
 	  s = strrchr(buf,'>');
 	  if (s) *s = 0;
 	  else return; /* No trailing '>' ? BAD BAD! */
-	  fprintf(mfp,"to %s\n",buf);
+	  fprintf(mfp,"to <%s>\n",buf);
 	} else {
 	  /*  Cc: some-address  */
-	  fprintf(mfp,"to %s\n",buf);
+	  fprintf(mfp,"to <%s>\n",buf);
 	}
 }
 
@@ -219,7 +218,7 @@ process(dp)
 
 	rp = dp->recipients;
 
-	fprintf(mfp, "to %s\n",rp->addr->link->user);
+	fprintf(mfp, "to <%s>\n",rp->addr->link->user);
 
 	/* copy error message file itself */
 	if ((mailshare = getzenv("MAILSHARE")) == NULL)
