@@ -3,8 +3,8 @@
  *	This will be free software, but only when it is finished.
  */
 
-#include <stdio.h>
 #include "hostenv.h"
+#include <stdio.h>
 #ifdef linux
 #define __USE_BSD 1
 #endif
@@ -15,16 +15,15 @@
 #include <sys/stat.h>
 #include "zmsignal.h"
 #include "malloc.h"
-#include NDIR_H
 
-#if	defined(USE_HOSTS) || defined(USE_RESOLV)
+#ifdef	HAVE_RESOLVER
 #include <netdb.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
-#endif	/* USE_HOSTS || USE_RESOLV */
+#endif
 #include "ta.h"
 
-#if	defined(TRY_AGAIN) && defined(USE_RESOLV)
+#if	defined(TRY_AGAIN) && defined(HAVE_RESOLVER)
 #define	BIND		/* Want BIND (named) nameserver support enabled */
 #endif	/* TRY_AGAIN */
 #ifdef	BIND
@@ -87,15 +86,15 @@ typedef char msgdata;
 #include "mail.h"
 #include "splay.h"
 
-#ifdef    USE_SETUIDX
-# include <sys/id.h>
-# define  USE_SETREUID
-# define  setreuid(x,y)   setuidx(ID_REAL|ID_EFFECTIVE, y)
+#if HAVE_SYS_WAIT_H /* POSIX-thing ?  If not, declare it so.. */
+# include <sys/wait.h>
+#endif
+#ifndef WEXITSTATUS
+# define WEXITSTATUS(stat_val) ((unsigned)(stat_val) >> 8)
+#endif
+#ifndef WIFEXITED
+# define WIFEXITED(stat_val) (((stat_val) & 255) == 0)
 #endif
-
-#ifdef	USE_UNIONWAIT
-#include <sys/wait.h>
-#endif	/* USE_UNIONWAIT */
 
 #ifndef	L_SET
 #define	L_SET	0
@@ -146,7 +145,7 @@ main(argc, argv)
 	char *channel, file[MAXPATHLEN+1];
 	int errflg, c, i;
 	struct ctldesc *dp;
-	SIGNAL_TYPE (*oldsig)();
+	RETSIGTYPE (*oldsig)();
 
 	pid = getpid();
 	cmdline = &argv[0][0];
@@ -280,9 +279,7 @@ process(dp)
 	if (lseek(dp->msgfd, dp->msgbodyoffset, L_SET) < 0L)
 	  warning("Cannot seek to message body! (%m)", (char *)NULL);
 
-#ifdef	USE_SETREUID
 	setreuid(-1, atoi(dp->senders->misc));
-#endif	/* USE_SETREUID */
 
 	if ((mfp = mail_open(MSG_RFC822)) == NULL) {
 	  for (rp = dp->recipients; rp != NULL; rp = rp->next)
@@ -293,14 +290,10 @@ process(dp)
 	      diagnostic(rp, EX_TEMPFAIL, 0,
 			 "cannot resubmit anything!");
 	    }
-#ifdef	USE_SETREUID
 	  setreuid(-1, getuid());
-#endif /* USE_SETREUID */
 	  return;
 	}
-#ifdef	USE_SETREUID
 	setreuid(-1, getuid());
-#endif	/* USE_SETREUID */
 
 	fprintf(mfp, "via suspension\n");
 	if (dp->senders->user[0] == 0)
@@ -525,7 +518,6 @@ int
 hold_ns(s)
 	char *s;
 {
-#ifdef	USE_HOSTS
 	struct qtypes *qtp;
 	char *cp, host[BUFSIZ];
 	struct hostent *hp;
@@ -560,11 +552,8 @@ hold_ns(s)
 	  break;
 	}
 	return hp != NULL;
-#else	/* !USE_HOSTS */
-	return 0;
-#endif	/* USE_HOSTS */
 }
-#endif	/* BIND */
+#endif	/* !BIND */
 
 /* return true if the seconds-since-epoch in argument has passed */
 
@@ -598,11 +587,7 @@ hold_script(command)
 	int i, in[2];
 	char *env[20], buf[8192], *cp, *s, *arg;
 	FILE *errfp;
-#ifdef	USE_UNIONWAIT
-	union wait status;
-#else	/* !USE_UNIONWAIT */
 	int status;
-#endif	/* USE_UNIONWAIT */
 
 	if ((arg = strchr(command, '/')) != NULL)
 	  *arg++ = '\0';
@@ -683,37 +668,21 @@ hold_script(command)
 	wait(&status);
 	fclose(errfp);
 	close(in[0]);
-#ifdef  USE_UNIONWAIT
-        if (status.w_termsig) {
+
+        if (WIFEXITED(status) > 0) {
 	  if (cp != errormsg)
 	    *cp++ = ' ';
-	  sprintf(cp, "[signal %d", status.w_termsig);
-	  if (status.w_coredump)
+	  sprintf(cp, "[signal %d", WIFEXITED(status) & 127);
+	  if (WIFEXITED(status)&0200)
 	    strcat(cp, " (Core dumped)");
 	  strcat(cp, "]");
 	  return 0;
-        } else if (status.w_retcode) {
+        } else if (WEXITSTATUS(status) > 0) {
 	  if (cp != errormsg)
 	    *cp++ = ' ';
-	  sprintf(cp, "[exit status %d]", status.w_retcode);
+	  sprintf(cp, "[exit status %d]", WEXITSTATUS(status));
 	  return 0;
         }
-#else   /* !USE_UNIONWAIT */
-        if ((status&0177) > 0) {
-	  if (cp != errormsg)
-	    *cp++ = ' ';
-	  sprintf(cp, "[signal %d", status&0177);
-	  if (status&0200)
-	    strcat(cp, " (Core dumped)");
-	  strcat(cp, "]");
-	  return 0;
-        } else if (((status>>8)&0377) > 0) {
-	  if (cp != errormsg)
-	    *cp++ = ' ';
-	  sprintf(cp, "[exit status %d]", (status>>8)&0377);
-	  return 0;
-        }
-#endif  /* USE_UNIONWAIT */
 	return 1;
 }
 
