@@ -13,8 +13,8 @@
  * Still more modifications by Matti Aarnio <mea@nic.funet.fi>
  */
 
-#include <stdio.h>
 #include "hostenv.h"
+#include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
 #include <pwd.h>
@@ -31,7 +31,25 @@
 #include "splay.h"
 #include "malloc.h"
 
-#ifdef	USE_LOCKF
+#ifdef  HAVE_WAITPID
+# include <sys/wait.h>
+#else
+# ifdef HAVE_WAIT3
+#  include <sys/wait.h> /* Has BSD wait3() */
+# else
+#  ifdef HAVE_SYS_WAIT_H /* POSIX.1 compatible */
+#   include <sys/wait.h>
+#  else /* Not POSIX.1 compatible, lets fake it.. */
+extern int wait();
+#  endif
+# endif
+#endif
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#ifdef	HAVE_LOCKF
 #ifdef	F_OK
 #undef	F_OK
 #endif	/* F_OK */
@@ -44,57 +62,30 @@
 #ifdef	R_OK
 #undef	R_OK
 #endif	/* R_OK */
-#include <unistd.h>
-#endif	/* USE_LOCKF */
+#endif	/* HAVE_LOCKF */
 
-#ifdef	USE_UNIONWAIT
-#include <sys/wait.h>
-#endif	/* USE_UNIONWAIT */
-
-#ifndef	USE_LSTAT
+#ifndef	HAVE_LSTAT
 #define	lstat stat
-#endif	/* !USE_LSTAT */
+#endif	/* !HAVE_LSTAT */
 
-#ifdef	USE_MAILLOCK
-# ifdef	MAILLOCK_H
-#  include MAILLOCK_H
-# else
-#  include <maillock.h>
-# endif
-#endif	/* USE_MAILLOCK */
+#ifdef	HAVE_MAILLOCK_H
+# include <maillock.h>
+#endif
 
-#ifdef USE_DOTLOCK
+#ifdef HAVE_DOTLOCK
 #include "dotlock.c"
 #endif
 
-#ifndef	USE_LSTAT
-#define	lstat stat
-#endif	/* !USE_LSTAT */
-
-#ifndef	USE_SETREUID
-#ifdef	USE_SETEUID
-#define	setreuid(x,y)	seteuid(y)
-#else
-#define	setreuid(x,y)	setuid(y)
-#endif	/* !USE_SETEUID */
-#endif	/* !USE_SETREUID */
-
-#ifdef  USE_SETUIDX	/* IBM AIXism */
-# undef  setreuid
-# include <sys/id.h>
-# define setreuid(x,y)   setuidx(ID_REAL|ID_EFFECTIVE, y)
-#endif  /* !USE_SETUIDX */
-
-#ifdef  USE_TIMEVAL
+#ifdef  HAVE_UTIMES
 #include <sys/time.h>
-#else   /* !USE_TIMEVAL */
-/*#include <unistd.h> */
+#else
+/* #include <unistd.h>  */
 /* some systems have utimbuf defined in unistd.h, some don't... */
 struct utimbuf {
 	time_t  actime;
 	time_t  modtime;
 };
-#endif  /* USE_TIMEVAL */
+#endif
 
 
 #ifndef	L_SET
@@ -107,25 +98,35 @@ struct utimbuf {
 #define	F_OK	0
 #endif	/* !F_OK */
 
-#include NDIR_H
+#ifdef HAVE_DIRENT_H
+# include <dirent.h>
+#else /* not HAVE_DIRENT_H */
+# define dirent direct
+# ifdef HAVE_SYS_NDIR_H
+#  include <sys/ndir.h>
+# endif /* HAVE_SYS_NDIR_H */
+# ifdef HAVE_SYS_DIR_H
+#  include <sys/dir.h>
+# endif /* HAVE_SYS_DIR_H */
+# ifdef HAVE_NDIR_H
+#  include <ndir.h>
+# endif /* HAVE_NDIR_H */
+#endif /* HAVE_DIRENT_H */
 
-#ifdef	BIFF
+#ifdef	HAVE_SOCKET
 #include <netdb.h>
 #include <sys/socket.h>
-#ifdef	SYS_STREAM_H
-#include SYS_STREAM_H
-#endif	/* SYS_STREAM_H */
 #include <netinet/in.h>
 #include <net/if.h>
-#endif	/* BIFF */
+#endif	/* HAVE_SOCKET */
 
-#ifdef RBIFF
+#ifdef HAVE_PROTOCOLS_RWHOD_H
 #include <utmp.h>
 #include <protocols/rwhod.h>
 #define RWHODIR		"/var/spool/rwho"
 #define	WHDRSIZE	(sizeof (wd) - sizeof (wd.wd_we))
 #define RBIFFRC		".rbiff"
-#endif	/* RBIFF */
+#endif	/* HAVE_PROTOCOLS_RWHOD_H */
 
 #define	PROGNAME	"mailbox"	/* for logging */
 #define	CHANNEL		"local"	/* the default channel name we deliver for */
@@ -179,7 +180,7 @@ char *maildirs[] = {
 					      EX_UNAVAILABLE : (E), 0, \
 						(A1), (A2), (A3))
 
-#if	defined(BIFF) || defined(RBIFF)
+#if	defined(HAVE_SOCKET)
 /*
  * Biff strategy:
  *
@@ -206,7 +207,7 @@ struct userhost {
 };
 
 struct sptree *spt_users = NULL;
-#endif	/* BIFF || RBIFF */
+#endif /* HAVE_SOCKET -- no use to biff, if no socket available .. */
 
 char *progname;
 char *channel;
@@ -218,6 +219,7 @@ uid_t currenteuid;		/* the current euid */
 extern int nobody;		/* safe uid for file/program delivery */
 int dobiff = 1;			/* enable biff notification */
 int dorbiff = 0;		/*    "   rbiff   " */
+int dorbiff_always = 0;
 int keepatime = 1;		/* preserve mbox st_atime, for login etc. */
 int creatflag = 1;		/* attempt to create files that don't exist */
 int mime8bit = 0;		/* Has code which translates  MIME text/plain
@@ -231,7 +233,7 @@ int mmdf_mode = 0;		/* Write out MMDF-style mail folder
 				   ("\001\001\001\001" as the separator..)  */
 long eofindex = -1;		/* When negative, putmail() can't truncate() */
 
-extern SIGNAL_TYPE wantout();
+extern RETSIGTYPE wantout();
 extern time_t time();
 extern int optind;
 extern char *optarg;
@@ -293,12 +295,12 @@ main(argc, argv)
 	int c, errflg, fd;
 	char *host = NULL;	/* .. and what is my host ? */
 	int matchhost = 0;
-#if	defined(BIFF) || defined(RBIFF)
+#if	defined(HAVE_SOCKET)
 	struct biffer *nbp;
 #endif
 	struct ctldesc *dp;
 
-	SIGNAL_TYPE (*oldsig)();
+	RETSIGTYPE (*oldsig)();
 
 	SIGNAL_HANDLESAVE(SIGINT, SIG_IGN, oldsig);
 	if (oldsig != SIG_IGN)
@@ -327,7 +329,7 @@ main(argc, argv)
 	errflg = 0;
 	logfile = NULL;
 	channel = CHANNEL;
-	while ((c = getopt(argc, argv, "abc:gh:l:rMV8")) != EOF) {
+	while ((c = getopt(argc, argv, "abc:gh:l:rRMV8")) != EOF) {
 	  switch (c) {
 	  case 'c':		/* specify channel scanned for */
 	    channel = optarg;
@@ -345,6 +347,9 @@ main(argc, argv)
 	  case 'r':		/* toggle rbiffing */
 	    dorbiff = !dorbiff;
 	    break;
+	  case 'R':
+	    dorbiff_always = !dorbiff_always;
+	    break;
 	  case 'a':		/* toggle mbox st_atime preservation */
 	    keepatime = !keepatime;
 	    break;
@@ -437,12 +442,12 @@ main(argc, argv)
 	  process(dp);
 	  ctlclose(dp);
 
-#if	defined(BIFF) || defined(RBIFF)
+#if	defined(HAVE_SOCKET)
 	  /* now that the delivery phase is over, hoot'n wave */
 	  for (nbp = biffs ; nbp != NULL; nbp = nbp->next) {
 	    if (dobiff && nbp->offset >= 0)
 	      biff("localhost", nbp->user, nbp->offset);
-#ifdef	RBIFF
+#ifdef	HAVE_PROTOCOLS_RWHOD_H /* RBIFF */
 	    if (nbp->wantrbiff != 0) {
 	      if (spt_users == NULL)
 		spt_users = sp_init();
@@ -451,7 +456,7 @@ main(argc, argv)
 	    }
 #endif /* RBIFF */
 	  }
-#ifdef	RBIFF
+#ifdef	HAVE_PROTOCOLS_RWHOD_H /* RBIFF */
 	  if (spt_users != NULL) {
 	    if (readrwho())
 	      for (nbp = biffs ; nbp != NULL; nbp = nbp->next) {
@@ -471,7 +476,7 @@ main(argc, argv)
 	    free(spt_users);
 	    spt_users = NULL;
 	  }
-#endif	/* BIFF || RBIFF */
+#endif	/* HAVE_SOCKET */
 	}
 	exit(EX_OK);
 	/* NOTREACHED */
@@ -482,7 +487,7 @@ void
 process(dp)
 	struct ctldesc *dp;
 {
-	char *cp, *ts, *at;
+	char *cp, *ts, *at = NULL, *plus = NULL;
 	struct rcpt *rp;
 	time_t curtime;
 	int is_mime_qptext = 0;
@@ -498,7 +503,8 @@ process(dp)
 	for (rp = dp->recipients; rp != NULL; rp = rp->next) {
 	  while (*(rp->addr->user) == TO_USER)
 	    rp->addr->user++;
-	  if ((at = strrchr(rp->addr->user,'@')) != NULL) *at = 0;
+	  if ((at   = strrchr(rp->addr->user,'@')) != NULL) *at = 0;
+	  if ((plus =  strchr(rp->addr->user,'+')) != NULL) *plus = 0;
 	  if (*(rp->addr->user) == QUOTE) {
 	    cp = rp->addr->user + strlen(rp->addr->user) - 1;
 	    if (*cp == QUOTE) {
@@ -507,13 +513,16 @@ process(dp)
 	    }
 	  }
 	  if (at) *at = '@';	/* If it followed quotes, a null prefixes it.. */
-#ifdef	RBIFF
+	  if (plus) *plus = '+';
+#ifdef  HAVE_SOCKET
+#ifdef	HAVE_PROTOCOLS_RWHOD_H /* RBIFF */
 	  /*
 	   * "contents" will be freed before rbiff done:
 	   * must copy username somewhere.
 	   */
 	  rp->addr->user = strdup(rp->addr->user);
 #endif /* RBIFF */
+#endif /* BIFF */
 	}
 
 	readalready = 0; /* ignore any previous message data cache */
@@ -591,22 +600,23 @@ deliver(dp, rp, timestring)
 	FILE *fp;
 	int hasdir;
 	struct stat st, s2;
-	char *file = NULL, *cp, *at;
+	char *file = NULL, *cp, *at, *plus;
 	int havedotlock = 0, havemaillock = 0;
 	int ismbox = 0;
-#if	defined(BIFF) || defined(RBIFF)
+#if	defined(HAVE_SOCKET)
 	struct biffer *nbp = NULL;
-#ifdef	RBIFF
+#ifdef	HAVE_PROTOCOLS_RWHOD_H
 	char *path;
-#endif	/* RBIFF */
-#endif	/* BIFF || RBIFF */
+#endif
+#endif
 	struct passwd *pw;
-#ifdef	USE_MAILLOCK
+#ifdef	HAVE_MAILLOCK_H
 	char *maillockuser;
 #endif
 
-	uid = atol(rp->addr->misc);
-	at = strchr(rp->addr->user,'@');
+	uid  = atol(rp->addr->misc);
+	at   = strchr(rp->addr->user,'@');
+	plus = strchr(rp->addr->user,'+');
 	switch (*(rp->addr->user)) {
 	case TO_PIPE:		/* pipe to program */
 	  /* one should disallow this if uid == nobody? */
@@ -658,9 +668,10 @@ deliver(dp, rp, timestring)
 	  break;
 	default:		/* local user */
 	  /* Zap the possible '@' for a moment -- and restore later
-	     [mea@utu.fi] */
+	     [mea@utu.fi] -- same with '+' --  user+localspecs */
 	  ismbox = 1;
-	  if (at) *at = 0;
+	  if (at)   *at   = 0;
+	  if (plus) *plus = 0;
 	  setpwent(); /* Effectively rewind the database,
 			 needed for multi-recipient processing ? */
 	  if ((pw = getpwnam(rp->addr->user)) == NULL) {
@@ -672,6 +683,7 @@ deliver(dp, rp, timestring)
 	    }
 	    if ((pw = getpwnam(rp->addr->user)) == NULL) {
 	      if (at) *at = '@';
+	      if (plus) *plus = '+';
 	      if (propably_x400(rp->addr->user)) {
 		notaryreport(rp->addr->user,"failed",
 			     "5.1.4 (this feels like a misplaced X.400 address -- no support for them)",
@@ -683,10 +695,12 @@ deliver(dp, rp, timestring)
 			     "5.1.1 (User does not exist)",
 			     "550 (User does not exist)");
 		if (at) *at = 0;
+		if (plus) *plus = '+';
 		DIAGNOSTIC(rp, EX_NOUSER,
 			   "user \"%s\" doesn't exist", rp->addr->user);
 	      }
 	      if (at) *at = '@';
+	      if (plus) *plus = '+';
 	      return;
 	    }
 	  }
@@ -729,23 +743,23 @@ deliver(dp, rp, timestring)
 	    closelog();
 	    return;
 	  }
-#if	defined(BIFF) || defined(RBIFF)
-	  if (!dobiff && !dorbiff)
+#if	defined(HAVE_SOCKET)
+	  if (!dobiff && !dorbiff && !dorbiff_always)
 	    break;
 	  nbp = (struct biffer *)emalloc(sizeof (struct biffer));
 	  nbp->next = NULL;
 	  nbp->user = strdup(rp->addr->user);
 	  nbp->offset = -1;
 	  nbp->wantrbiff = 0;
-#ifdef	RBIFF
+#ifdef	HAVE_PROTOCOLS_RWHOD_H
 	  if (dorbiff) {
-#ifdef	RBIFF_ALWAYS
-	    nbp->wantrbiff = 1;
-#else
-	    path = emalloc(2+strlen(pw->pw_dir)+strlen(RBIFFRC));
-	    sprintf(path, "%s/%s", pw->pw_dir, RBIFFRC);
-	    nbp->wantrbiff = (access(path, F_OK) == 0);
-#endif /* RBIFF_ALWAYS */
+	    if (dorbiff_always)
+	      nbp->wantrbiff = 1;
+	    else {
+	      path = emalloc(2+strlen(pw->pw_dir)+strlen(RBIFFRC));
+	      sprintf(path, "%s/%s", pw->pw_dir, RBIFFRC);
+	      nbp->wantrbiff = (access(path, F_OK) == 0);
+	    }
 	  }
 #endif /* RBIFF */
 	  if (biffs == NULL)
@@ -755,7 +769,8 @@ deliver(dp, rp, timestring)
 	    eobiffs = nbp;
 	  }
 #endif /* BIFF || RBIFF */
-	  if (at) *at = '@';
+	  if (at)   *at   = '@';
+	  if (plus) *plus = '+';
 	  break;
 	}
 	if (exstat(rp, file, &st, lstat) < 0) {
@@ -865,7 +880,7 @@ deliver(dp, rp, timestring)
 	if (!S_ISREG(st.st_mode))
 	  /* don't lock non-files */;
 	else {
-#ifdef	USE_MAILLOCK
+#ifdef	HAVE_MAILLOCK_H
 	  if (*(rp->addr->user) == TO_FILE) {
 	    if ((pw = getpwuid(uid)) == NULL) {
 	      notaryreport(file,"failed",
@@ -921,8 +936,8 @@ deliver(dp, rp, timestring)
 	    }
 	    havemaillock = 1;
 	  }
-#endif	/* USE_MAILLOCK */
-#ifdef  USE_DOTLOCK
+#endif	/* HAVE_MAILLOCK_H */
+#ifdef  HAVE_DOTLOCK
 	  if (ismbox &&
 	      !(havedotlock = dotlock(file) == 0)) {
 	    char mbuf[256];
@@ -930,24 +945,24 @@ deliver(dp, rp, timestring)
 	    DIAGNOSTIC(rp, errno == EBUSY ? EX_TEMPFAIL :
 		       (errno == EACCES ? EX_NOPERM : EX_UNAVAILABLE),
 		       "can't lock %s", mbuf);
-#ifdef	USE_MAILLOCK
+#ifdef	HAVE_MAILLOCK_H
 	    if (havemaillock)
 	      mailunlock();
 #endif
 	    return;
 	  }
-#endif  /* USE_DOTLOCK */
+#endif  /* HAVE_DOTLOCK */
 #ifdef	USE_NFSMBOX
 	  if (nfslock(file, LOCK_EX) != 0) {
 	    alarm(0);
-#ifdef	USE_MAILLOCK
+#ifdef	HAVE_MAILLOCK_H
 	    if (havemaillock)
 	      mailunlock();
 #endif
-#ifdef  USE_DOTLOCK
+#ifdef  HAVE_DOTLOCK
 	    if (havedotlock)
 	      dotunlock(file);
-#endif /* USE_DOTLOCK */
+#endif /* HAVE_DOTLOCK */
 	    notaryreport(file,"delayed",
 			 "4.4.5 (File locking with NFS lock failed)",
 			 "450 (File locking with NFS lock failed)");
@@ -958,19 +973,19 @@ deliver(dp, rp, timestring)
 	    return;
 	  }
 #endif	/* USE_NFSMBOX */
-#ifdef	USE_LOCKF
+#ifdef	HAVE_LOCKF
 	  /* Seek to begining for locking! [mea@utu.fi] */
 	  if (lseek(fdmail,0L,L_SET) < 0 ||
 	      lockf(fdmail, F_LOCK, 0) < 0) {
 	    alarm(0);
-#ifdef	USE_MAILLOCK
+#ifdef	HAVE_MAILLOCK_H
 	    if (havemaillock)
 	      mailunlock();
 #endif
-#ifdef  USE_DOTLOCK
+#ifdef  HAVE_DOTLOCK
 	    if (havedotlock)
 	      dotunlock(file);
-#endif /* USE_DOTLOCK */
+#endif /* HAVE_DOTLOCK */
 	    notaryreport(file,"delayed",
 			 "4.4.5 (File locking with  lockf  failed)",
 			 "450 (File locking with  lockf  failed)");
@@ -981,18 +996,18 @@ deliver(dp, rp, timestring)
 	    return;
 	  }
 
-#endif	/* USE_LOCKF */
-#ifdef	USE_FLOCK
+#endif	/* HAVE_LOCKF */
+#ifdef	HAVE_FLOCK
 	  if (flock(fdmail, LOCK_EX) < 0) {
 	    alarm(0);
-#ifdef	USE_MAILLOCK
+#ifdef	HAVE_MAILLOCK_H
 	    if (havemaillock)
 	      mailunlock();
 #endif
-#ifdef  USE_DOTLOCK
+#ifdef  HAVE_DOTLOCK
 	    if (havedotlock)
 	      dotunlock(file);
-#endif /* USE_DOTLOCK */
+#endif /* HAVE_DOTLOCK */
 	    notaryreport(file,"delayed",
 			 "4.4.5 (File locking with  flock  failed)",
 			 "450 (File locking with  flock  failed)");
@@ -1002,7 +1017,7 @@ deliver(dp, rp, timestring)
 	    close(fdmail);
 	    return;
 	  }
-#endif	/* USE_LOCKF */
+#endif	/* HAVE_LOCKF */
 	}
 
 	/* Turn off the alarm */
@@ -1011,7 +1026,7 @@ deliver(dp, rp, timestring)
 
 	/* Where is the end of file -- before we write anything */
 	eofindex = lseek(fdmail, 0L, L_XTND);
-#if	defined(BIFF) || defined(RBIFF)
+#if	defined(HAVE_SOCKET)
 	if (nbp != NULL)
 	  nbp->offset = eofindex;
 #endif	/* BIFF || RBIFF */
@@ -1021,23 +1036,23 @@ deliver(dp, rp, timestring)
 #ifdef	USE_NFSMBOX
 	  unlock(file);
 #endif	/* USE_NFSMBOX */
-#ifdef	USE_LOCKF
+#ifdef	HAVE_LOCKF
 	  lseek(fdmail,0L,L_SET);
 	  lockf(fdmail, F_ULOCK, 0);/* XX: does this really work?*/
 	  /* Yes it does, but it needs seek to begining of
 	     the file at first! [mea@utu.fi] */
-#endif	/* USE_LOCKF */
-#if	defined(USE_FLOCK) || (!defined(USE_NFSMBOX) && !defined(USE_LOCKF) && !defined(USE_MAILLOCK))
+#endif	/* HAVE_LOCKF */
+#if	defined(HAVE_FLOCK) || (!defined(USE_NFSMBOX) && !defined(HAVE_LOCKF) && !defined(HAVE_MAILLOCK_H))
 	  flock(fdmail, LOCK_UN);
-#endif	/* USE_FLOCK */
-#ifdef	USE_MAILLOCK
+#endif	/* HAVE_FLOCK */
+#ifdef	HAVE_MAILLOCK_H
 	  if (havemaillock)
 	    mailunlock();
-#endif	/* USE_MAILLOCK */
-#ifdef	USE_DOTLOCK
+#endif	/* HAVE_MAILLOCK_H */
+#ifdef	HAVE_DOTLOCK
 	  if (havedotlock)
 	    dotunlock(file);
-#endif /*USE_DOTLOCK*/
+#endif /*HAVE_DOTLOCK*/
 	}
 	setrootuid(rp);
 	if (fp != NULL) {
@@ -1047,7 +1062,7 @@ deliver(dp, rp, timestring)
 		       "250 (Delivered successfully)");
 	  DIAGNOSTIC(rp, EX_OK, (char *)NULL, 0);
 	}
-#if	defined(BIFF) || defined(RBIFF)
+#if	defined(HAVE_SOCKET)
 	else
 	  if (nbp != NULL) /* putmail() has produced a DIAGNOSTIC */
 	    nbp->offset = -1;
@@ -1079,60 +1094,24 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	/* append From_ line and print out the header */
 
 	if (!mmdf_mode &&
-	    *(rp->addr->user) != TO_PIPE && eofindex > 0L &&
-	    fseek(fp, -2L, L_XTND) != -1) {
-	  /*
-	   * Determine how many newlines to append to previous text.
-	   *
-	   * If the last characters are:	\n\n, append 0 newlines,
-	   *				[^\n]\n or \n append 1 newline,
-	   *				else append 2 newlines.
-	   *
-	   * Optionally preserve the mbox's atime, so
-	   * login etc. can distinguish new mail from old.
-	   * The mtime will be set to now by the following write() calls.
-	   */
+	    *(rp->addr->user) != TO_PIPE && eofindex > 0L) {
 
 	  fstat(FILENO(fp), &st);
 	  if (!keepatime) st.st_atime = 0;
 
-	  len = fread(buf, 1, 2, fp);
-	  fseek(fp, 0L, L_XTND);	/* to end of file, again */
-
 	  if (st.st_atime != 0) {
-#ifdef	USE_TIMEVAL
+#ifdef	HAVE_UTIMES
 	    struct timeval tv[2];
 	    tv[0].tv_sec = st.st_atime;
 	    tv[1].tv_sec = st.st_mtime;
 	    tv[0].tv_usec = tv[1].tv_usec = 0;
 	    rc = utimes(file, tv);
-#else  /* !USE_TIMEVAL */
+#else  /* !HAVE_UTIMES */
 	    struct utimbuf tv;
 	    tv.actime  = st.st_atime;
 	    tv.modtime = st.st_mtime;
 	    rc = utime(file, &tv);
-#endif /* USE_TIMEVAL */
-	  }
-
-	  if (len == 1 || len == 2) {
-	    --len;
-	    len = (buf[len]!='\n') + (len == 1 ? buf[0]!='\n' : 1);
-	    if (len > 0 && (fwrite("\n\n", 1, len, fp) != len
-			    || fflush(fp) == EOF)) {
-
-	      notaryreport(NULL,NULL,NULL,NULL);
-	      DIAGNOSTIC(rp, EX_IOERR,
-			 "cleansing of \"%s\" failed", file);
-	      /* XX: should I really do this? */
-#ifdef	USE_FTRUNCATE
-	      fflush(fp);
-	      if (eofindex >= 0)
-		ftruncate(FILENO(fp), (u_long)eofindex);
-#endif /* USE_FTRUNCATE */
-	      fclose(fp);
-	      fp = NULL;
-	      goto time_reset;
-	    }
+#endif /* !HAVE_UTIMES */
 	  }
 	} else if (eofindex > 0L && eofindex < (sizeof "From x\n")) {
 	  /* A mail message *cannot* be this small.  It must be
@@ -1160,47 +1139,75 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 		       "500 (Write to user's mailbox failed)");
 	  DIAGNOSTIC(rp, EX_IOERR,
 		     "header write to \"%s\" failed", file);
+#ifdef HAVE_FTRUNCATE
 	  /* XX: should I really do this? */
 	  fflush(fp);
 	  if (eofindex >= 0)
 	    ftruncate(FILENO(fp), (u_long)eofindex);
+#endif
 	  fclose(fp);
 	  fp = NULL;
 	  goto time_reset;
 	}
 	lastch = appendlet(dp, rp, fp, file, is_mime);
 
-	if (lastch == EOF) {
+	if (lastch == EOF || ferror(fp) || fflush(fp) == EOF) {
+      write_failure:
+	  notaryreport(NULL,NULL,NULL,NULL);
+	  DIAGNOSTIC3(rp, EX_IOERR,
+		      "message write to \"%s\" failed: %s",
+		      file, strerror(errno));
+#ifdef HAVE_FTRUNCATE
 	  /* XX: should I really do this? */
 	  fflush(fp);
 	  if (eofindex >= 0)
 	    ftruncate(FILENO(fp), (u_long)eofindex);
+#endif
 	  fclose(fp);
 	  fp = NULL;
 	  goto time_reset;
 	}
 
-	/* End of the file, and MMDF-style ? */
-	if (mmdf_mode == 1 /* Values 2 and 3 exist on PIPEs only.. */   )
-	  fputs("\001\001\001\001\n",fp);
-
-	/* ---- knock-off unnecessary trailing newlines ????      ---
-	   ---- if lastch == '\n', adding ONE newline is enough ? ---
-	   if (lastch != '\n' && !ferror(fp))
-	   fputc('\n', fp);
+	/*
+	 * Determine how many newlines to append to the previous text.
+	 *
+	 * If the last characters are:	\n\n, append 0 newlines,
+	 *				[^\n]\n or \n append 1 newline,
+	 *				else append 2 newlines.
+	 *
+	 * Optionally preserve the mbox's atime, so
+	 * login etc. can distinguish new mail from old.
+	 * The mtime will be set to now by the following write() calls.
 	 */
-	if (fputc('\n', fp) == EOF || fflush(fp) == EOF) {
-	  notaryreport(NULL,NULL,NULL,NULL);
-	  DIAGNOSTIC3(rp, EX_IOERR,
-		      "message write to \"%s\" failed: %s",
-		      file, strerror(errno));
-	  /* XX: should I really do this? */
-	  fflush(fp);
-	  if (eofindex >= 0)
-	    ftruncate(FILENO(fp), (u_long)eofindex);
-	  fclose(fp);
-	  fp = NULL;
-	  goto time_reset;
+	fseek(fp, -2L, L_XTND);
+	len = fread(buf, 1, 2, fp);
+	fseek(fp, 0L, L_XTND);	/* to end of file, again */
+
+	if (len == 1 || len == 2) {
+	  --len;
+	  len = (buf[len]!='\n') + (len == 1 ? buf[0]!='\n' : 1);
+	  if (len > 0 && (fwrite("\n\n", 1, len, fp) != len
+			  || fflush(fp) == EOF)) {
+
+	    notaryreport(NULL,NULL,NULL,NULL);
+	    DIAGNOSTIC(rp, EX_IOERR,
+		       "cleansing of \"%s\" failed", file);
+#ifdef	HAVE_FTRUNCATE
+	    /* XX: should I really do this? */
+	    fflush(fp);
+	    if (eofindex >= 0)
+	      ftruncate(FILENO(fp), (u_long)eofindex);
+#endif /* HAVE_FTRUNCATE */
+	    fclose(fp);
+	    fp = NULL;
+	    goto time_reset;
+	  }
+	}
+
+	/* End of the file, and MMDF-style ? */
+	if (mmdf_mode == 1 /* Values 2 and 3 exist on PIPEs only.. */ ) {
+	  if (fputs("\001\001\001\001\n",fp) == EOF)
+	    goto write_failure;
 	}
 
 	/* This is actually Linux-thing, we reset the last access time
@@ -1228,18 +1235,18 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
       time_reset:
 
 	if (st.st_atime != 0) {
-#ifdef	USE_TIMEVAL
+#ifdef	HAVE_UTIMES
 	  struct timeval tv[2];
 	  tv[0].tv_sec = st.st_atime;
 	  tv[1].tv_sec = st.st_mtime;
 	  tv[0].tv_usec = tv[1].tv_usec = 0;
 	  rc = utimes(file, tv);
-#else  /* !USE_TIMEVAL */
+#else  /* !HAVE_UTIMES */
 	  struct utimbuf tv;
 	  tv.actime  = st.st_atime;
 	  tv.modtime = st.st_mtime;
 	  rc = utime(file, &tv);
-#endif /* USE_TIMEVAL */
+#endif /* !HAVE_UTIMES */
 	}
 
 
@@ -1256,11 +1263,7 @@ program(dp, rp, timestring)
 	uid_t uid;
 	gid_t gid;
 	char *env[20], buf[8192], *cp, *s;
-#ifdef	USE_UNIONWAIT
-	union wait status;
-#else	/* !USE_UNIONWAIT */
 	int status;
-#endif	/* USE_UNIONWAIT */
 	struct passwd *pw;
 	FILE *errfp;
 	FILE *fp;
@@ -1390,23 +1393,9 @@ program(dp, rp, timestring)
 	fclose(errfp);
 	close(in[0]);
 	cp = buf + strlen(buf);
-#ifdef	USE_UNIONWAIT
-	if (status.w_termsig) {
-	  if (cp != buf)
-	    *cp++ = ' ';
-	  sprintf(cp, "[signal %d", status.w_termsig);
-	  if (status.w_coredump)
-	    strcat(cp, " (Core dumped)");
-	  strcat(cp, "]");
-	  i = EX_TEMPFAIL;
-	} else if (status.w_retcode) {
-	  if (cp != buf)
-	    *cp++ = ' ';
-	  sprintf(cp, "[exit status %d]", status.w_retcode);
-	  i = EX_UNAVAILABLE;
-	} else
-		i = EX_OK;
-#else	/* !USE_UNIONWAIT */
+
+	/* Union or not, we treat it as if it were an integer.. */
+
 	if ((status&0177) > 0) {
 	  if (cp != buf)
 	    *cp++ = ' ';
@@ -1422,7 +1411,7 @@ program(dp, rp, timestring)
 	  i = EX_UNAVAILABLE;
 	} else
 		i = EX_OK;
-#endif	/* USE_UNIONWAIT */
+
 	notaryreport(NULL,NULL,NULL,NULL); /* XX: proper values ? */
 	DIAGNOSTIC(rp, i, "%s", buf);
 	return;
@@ -1451,15 +1440,15 @@ creatembox(rp, filep, uid, gid, pw)
 	  *filep = emalloc(2+strlen(*maild)+strlen(rp->addr->user));
 	  sprintf(*filep, "%s/%s", *maild, rp->addr->user);
 	  if ((fd = createfile(rp, *filep, *uid)) >= 0) {
-#ifdef	USE_FCHOWN
+#ifdef	HAVE_FCHOWN
 	    fchown(fd, *uid, *gid);
-#else  /* !USE_FCHOWN */
+#else  /* !HAVE_FCHOWN */
 	    chown(*filep, *uid, *gid);
-#endif /* USE_FCHOWN */
+#endif /* HAVE_FCHOWN */
 	    close(fd);
 	    {
 	      struct stat st;
-#ifdef	USE_TIMEVAL
+#ifdef	HAVE_UTIMES
 	      struct timeval tv[2];
 	      stat(*filep,&st); /* This by all propability will not fail.. */
 	      tv[0].tv_sec = 1; /* Not zero, but close.. */
@@ -1652,7 +1641,7 @@ appendlet(dp, rp, fp, file, ismime)
 {
 	int lastch = 0xFFF; /* Something no character can be,
 			       and not "-1" either.. */
-#ifndef	USE_MMAP
+#ifndef	HAVE_MMAP
 	static char buffer[BUFSIZ+1];
 	static char *bufend = buffer+BUFSIZ;
 	register int i;
@@ -1758,7 +1747,7 @@ appendlet(dp, rp, fp, file, ismime)
 	  if (bufferfull > 1)	/* not all in memory, need to reread */
 	    readalready = 0;
 	}
-#else /* USE_MMAP  --  get the input from  mmap():ed memory area.. */
+#else /* HAVE_MMAP  --  get the input from  mmap():ed memory area.. */
 	char *s; /* message body start */
 	s = dp->let_buffer + dp->msgbodyoffset;
 	if (ismime) {
@@ -1810,7 +1799,7 @@ exstat(rp, file, stbufp, statfcn)
 	return 0;
 }
 
-#if	defined(BIFF) || defined(RBIFF)
+#if	defined(HAVE_SOCKET)
 void
 biff(hostname, username, offset)
 	char *hostname, *username;
@@ -1856,7 +1845,7 @@ biff(hostname, username, offset)
 #endif	/* BIFF || RBIFF */
 
 
-#ifdef RBIFF
+#if defined(HAVE_SOCKET) && defined(HAVE_PROTOCOLS_RWHOD_H)
 int
 readrwho()
 {
@@ -1874,7 +1863,7 @@ readrwho()
 	long now;
 	char username[NMAX+1];
 	DIR *dirp, *opendir();
-	struct NDIR_TYPE *dp, *readdir();
+	struct dirent *dp, *readdir();
 	struct userhost *uhp;
 	struct spblk *spl;
 	
