@@ -17,10 +17,30 @@
 #include "mail.h"
 #include "malloc.h"
 #include "ta.h"
-#include NDIR_H
-#ifdef	USE_UNIONWAIT
-#include <sys/wait.h>
-#endif	/* USE_UNIONWAIT */
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#ifdef  HAVE_WAITPID
+# include <sys/wait.h>
+#else
+# ifdef HAVE_WAIT3
+#  include <sys/wait.h> /* Has BSD wait3() */
+# else
+#  ifdef HAVE_SYS_WAIT_H /* POSIX.1 compatible */
+#   include <sys/wait.h>
+#  else /* Not POSIX.1 compatible, lets fake it.. */
+extern int wait();
+#  endif
+# endif
+#endif
+
+#ifndef WIFEXITED
+# define WIFEXITED(s) (((s) >> 8) & 0377)
+#endif
+#ifndef WIFSIGNALED
+# define WIFSIGNALED(s) ((s) & 0177)
+#endif
 
 #ifndef	L_SET
 #define	L_SET	0
@@ -47,7 +67,7 @@ int	maxwidth = 0;
 
 int	D_alloc = 0;		/* Memory debugging */
 
-extern SIGNAL_TYPE wantout();
+extern RETSIGTYPE wantout();
 #ifndef MALLOC_TRACE
 extern univptr_t emalloc();
 extern univptr_t erealloc();
@@ -130,7 +150,7 @@ main(argc, argv)
 	struct ctldesc *dp;
 	int errflg, c;
 	struct maildesc *mp;
-	SIGNAL_TYPE (*oldsig)();
+	RETSIGTYPE (*oldsig)();
 
 	SIGNAL_HANDLESAVE(SIGINT, SIG_IGN, oldsig);
 	if (oldsig != SIG_IGN)
@@ -293,11 +313,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	FILE *tafp, *errfp;
 	char *cp, *ds, *s, buf[BUFSIZ], buf2[BUFSIZ];
 	char **av;
-#ifdef	USE_UNIONWAIT
-	union wait status;
-#else	/* !USE_UNIONWAIT */
 	int	status;
-#endif	/* USE_UNIONWAIT */
 	int content_kind, conversion_prohibited, convertmode, ascii_clean = 0;
 
 	if (lseek(dp->msgfd, dp->msgbodyoffset, L_SET) < 0L)
@@ -536,7 +552,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  kill(pid, SIGTERM);
 	  sleep(1);
 	  kill(pid, SIGKILL);
-	  wait((union wait *)0);
+	  wait(NULL);
 	  return;
 	}
 	fclose(tafp);
@@ -549,44 +565,25 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	close(in[0]);	/* more paranoia */
 	pid = wait(&status);
 	cp = buf + strlen(buf);
-#ifdef	USE_UNIONWAIT
-	if (status.w_termsig) {
-	  if (cp != buf)
-	    *cp++ = ' ';
-	  sprintf(cp, "[signal %d", status.w_termsig);
-	  if (status.w_coredump)
-	    strcat(cp, " (Core dumped)");
-	  strcat(cp, "]");
-	  i = EX_TEMPFAIL;
-	} else if (status.w_retcode == 0 || status.w_retcode == EX_OK) {
-	  i = EX_OK;
-	} else {
-	  i = status.w_retcode;
-#else  /* !USE_UNIONWAIT */
-	  if (status&0177) {
+	  if (WIFSIGNALED(status)) {
 	    if (cp != buf)
 	      *cp++ = ' ';
-	    sprintf(cp, "[signal %d", status&0177);
+	    sprintf(cp, "[signal %d", WIFSIGNALED(status));
 	    if (status&0200)
 	      strcat(cp, " (Core dumped)");
 	    strcat(cp, "]");
 	    i = EX_TEMPFAIL;
-	  } else if (((status>>8)&0377) == 0 || ((status>>8)&0377) == EX_OK) {
+	  } else if (WIFEXITED(status) == 0 || WIFEXITED(status) == EX_OK) {
 	    i = EX_OK;
 	  } else {
-	    i = (status>>8)&0377;
-#endif /* USE_UNIONWAIT */
+	    i = WIFEXITED(status);
 	    for (exp = exmap; exp->origstatus != 0; ++exp)
 	      if (exp->origstatus == i)
 		break;
 
 	    s = exp->statusmsg;
 	    i = exp->newstatus;
-#ifdef	USE_UNIONWAIT
-	    sprintf(cp, "[exit status %d", status.w_retcode);
-#else  /* !USE_UNIONWAIT */
-	    sprintf(cp, "[exit status %d", ((status>>8)&0377));
-#endif /* USE_UNIONWAIT */
+	    sprintf(cp, "[exit status %d", WIFEXITED(status));
 	    if (s)
 	      sprintf(cp+strlen(cp), " (%s)", s);
 	    sprintf(cp+strlen(cp), " of command: %s", mp->command);
@@ -612,7 +609,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
  * appendlet - append letter to file pointed at by fd
  */
 
-#ifndef	USE_MMAP
+#ifndef	HAVE_MMAP
 static char let_buffer[BUFSIZ];
 #endif
 
@@ -634,13 +631,13 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	register int i;
 	register int bufferfull;
 	int lastch;
-#ifndef USE_MMAP
+#ifndef HAVE_MMAP
 	int mfd = dp->msgfd;
 #endif
 
 	writebuf(mp, fp, (char *)NULL, 0);  /* magic initialization */
 
-#ifndef USE_MMAP
+#ifndef HAVE_MMAP
 	/* can we use cache of message body data */
 	if (convertmode == _CONVERT_NONE && readalready != 0) {
 	  lastch = let_buffer[readalready-1];
@@ -657,7 +654,7 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	bufferfull = 0;
 	lastch = -1;
 	if (convertmode == _CONVERT_NONE) {
-#ifndef	USE_MMAP
+#ifndef	HAVE_MMAP
 	  while ((i = read(mfd, let_buffer, sizeof let_buffer)) != 0) {
 #else
 	    char *let_buffer = dp->let_buffer + dp->msgbodyoffset;
@@ -670,7 +667,7 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	      return EX_IOERR;
 	    readalready = i;
 	    bufferfull++;
-#ifndef	USE_MMAP
+#ifndef	HAVE_MMAP
 	  }
 #endif
 	} else {
@@ -678,7 +675,7 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	  /* Various esoteric conversion modes..
 	     We are better to feed writemimeline() with LINES
 	     instead of blocks of data.. */
-#ifndef	USE_MMAP
+#ifndef	HAVE_MMAP
 	  char iobuf[BUFSIZ];
 	  FILE *mfp = fdopen(mfd,"r");
 
@@ -709,7 +706,7 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	  i = 0;
 
 	  for (;;) {
-#ifndef USE_MMAP
+#ifndef HAVE_MMAP
 	    if ((i = cfgets(let_buffer, sizeof(let_buffer), mfp)) == EOF)
 	      break;
 #else
@@ -730,7 +727,7 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	      return EX_IOERR;
 	    }
 	  }
-#ifndef USE_MMAP
+#ifndef HAVE_MMAP
 	  if (i == EOF && !feof(mfp)) {
 	    MFPCLOSE
 	    return EX_IOERR;
@@ -743,7 +740,7 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	if (lastch != '\n')
 	  writebuf(mp, fp, "\n", 1);
 
-#ifndef USE_MMAP
+#ifndef HAVE_MMAP
 	if (bufferfull > 1)	/* not all in memory, need to reread */
 	  readalready = 0;
 #endif
@@ -1111,7 +1108,7 @@ int
 check_7bit_cleanness(dp)
 struct ctldesc *dp;
 {
-#ifdef	USE_MMAP
+#ifdef	HAVE_MMAP
 	/* With MMAP()ed spool file it is sweet and simple.. */
 	register char *s = dp->let_buffer + dp->msgbodyoffset;
 	while (s < dp->let_end)
