@@ -24,8 +24,8 @@
 
 			 */
 
-#include <stdio.h>
 #include "hostenv.h"
+#include <stdio.h>
 #ifdef linux
 #define __USE_BSD 1
 #endif
@@ -40,10 +40,6 @@
 #include <sys/file.h>
 #include <sys/param.h>
 #include <sys/stat.h>
-#include NDIR_H
-#if defined(USE_ALLOCA) && defined(ALLOCA_H)
-# include ALLOCA_H
-#endif
 
 #include <netdb.h>
 #include <sys/socket.h>
@@ -52,21 +48,7 @@
 #include "ta.h"
 #include "malloc.h"
 
-#ifndef	USE_SETREUID
-#ifdef	USE_SETEUID
-#define	setreuid(x,y)	seteuid(y)
-#else
-#define	setreuid(x,y)	setuid(y)
-#endif	/* !USE_SETEUID */
-#endif	/* !USE_SETREUID */
-
-#ifdef  USE_SETUIDX	/* IBM AIXism */
-# undef  setreuid
-# include <sys/id.h>
-# define setreuid(x,y)   setuidx(ID_REAL|ID_EFFECTIVE, y)
-#endif  /* !USE_SETUIDX */
-
-#if	defined(TRY_AGAIN) && defined(USE_RESOLV)
+#if	defined(TRY_AGAIN) && defined(HAVE_RESOLVER)
 #define	BIND		/* Want BIND (named) nameserver support enabled */
 #endif	/* TRY_AGAIN */
 #ifdef	BIND
@@ -177,10 +159,11 @@ char *myaddrs = NULL;
 int nmyaddrs = 0;
 int pid;
 int debug = 0;
+int verbosity = 0;
 int conndebug = 0;
 int timeout = 0;		/* how long do we wait for response? (sec.) */
 int gotalarm =0;		/* indicate that alarm happened! */
-#ifndef	USE_MMAP
+#ifndef	HAVE_MMAP
 int readalready = 0;		/* does buffer contain valid message data? */
 #endif
 int wantreserved = 0;		/* open connection on secure (reserved) port */
@@ -191,6 +174,7 @@ int mxcount = 0;
 int force_8bit = 0;		/* Claim to the remote to be 8-bit system, even
 				   when it doesn't report itself as such..*/
 int force_7bit = 0;		/* and reverse the previous.. */
+int keep_header8 = 0;		/* Don't do "MIME-2" to the headers */
 int checkwks = 0;
 int headers_contain_8bit = 0;
 FILE *logfp = NULL;
@@ -285,8 +269,8 @@ extern int getmxrr __((char*,struct mxdata*,int,int,int*));
 # endif /* RFC974 */
 #endif	/* BIND */
 extern int matchroutermxes __((char*,struct address*));
-extern SIGNAL_TYPE sig_pipe __((int));
-extern SIGNAL_TYPE sig_alarm __((int));
+extern RETSIGTYPE sig_pipe __((int));
+extern RETSIGTYPE sig_alarm __((int));
 extern int getmyhostname();
 extern void stashmyaddresses();
 extern int cistrcmp(), cistrncmp();
@@ -337,7 +321,7 @@ main(argc, argv, envp)
 #ifdef	BIND
 	int checkmx = 0;	/* check all destination hosts for MXness */
 #endif	/* BIND */
-	SIGNAL_TYPE (*oldsig)();
+	RETSIGTYPE (*oldsig)();
 
 	smtpfp[0] = NULL;
 	smtpfp[1] = NULL;
@@ -381,7 +365,7 @@ main(argc, argv, envp)
 	myhostname[0] = '\0';
 	remotemsg[0] = '\0';
 	remotehost[0] = '\0';
-	while ((c = getopt(argc, argv, "c:deh:l:p:PrsxDET:VW78")) != EOF) {
+	while ((c = getopt(argc, argv, "c:deh:l:p:rsvxDEHPT:VW78")) != EOF) {
 	  switch (c) {
 	  case 'c':		/* specify channel scanned for */
 	    channel = strdup(optarg);
@@ -435,6 +419,9 @@ main(argc, argv, envp)
 	      ++errflg;
 	    }
 	    break;
+	  case 'v':
+	    ++verbosity;
+	    break;
 	  case 'V':
 	    prversion("smtp");
 	    exit(0);
@@ -442,6 +429,9 @@ main(argc, argv, envp)
 	  case 'W':		/* Enable RFC974 WKS checks */
 	    checkwks = 1;
 	    break;
+	  case 'H':
+	    keep_header8 = 1;
+	    break;
 	  case '8':
 	    force_8bit = 1;
 	    force_7bit = 0;
@@ -456,7 +446,7 @@ main(argc, argv, envp)
 	}
 	if (errflg || optind > argc) {
 	  fprintf(stderr,
-		  "Usage: %s [-c channel] [host]\n", argv[0]);
+		  "Usage: %s [-8|-8H|-7][-e][-r][-x][-E][-P][-W][-T timeout][-h myhostname][-l logfile][-p portnum][-c channel] [host]\n", argv[0]);
 	  exit(EX_USAGE);
 	}
 	if (optind < argc) {
@@ -540,7 +530,7 @@ main(argc, argv, envp)
 	      if (now < retryat_time) {
 		/* Same host, we don't touch on it for a while.. */
 		sleep(2);
-		if (logfp) {
+		if (logfp && verbosity > 1) {
 		  fprintf(logfp,"%s#\t(too soon trying to touch on host with 'retryat' diagnostic -- flushing job queue..host='%s')\n",logtag(),host);
 		}
 		continue;
@@ -558,6 +548,7 @@ main(argc, argv, envp)
 		if (logfp) {
 		  fprintf(logfp, "%s#\t(closed SMTP channel - new host)\n", logtag());
 		}
+		strcpy(remotehost, host);
 		smtp_isopen = 0;
 	      }
 	    }
@@ -662,7 +653,7 @@ process(dp, smtpstatus, host, smtpfp, verboselog)
 	int loggedid;
 	int openstatus = EX_OK;
 
-#ifndef	USE_MMAP
+#ifndef	HAVE_MMAP
 	readalready = 0; /* ignore any previous message data cache */
 #endif
 	loggedid = 0;
@@ -801,7 +792,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	  } /* switch().. */
 
 	  hdr_mime2 = headers_need_mime2(startrp);
-	  if (hdr_mime2) {
+	  if (hdr_mime2 && !keep_header8) {
 	    headers_to_mime2(startrp,verboselog);
 	  }
 
@@ -1076,7 +1067,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
  * appendlet - append letter to file pointed at by fd
  */
 
-#ifndef	USE_MMAP
+#ifndef	HAVE_MMAP
 static char let_buffer[BUFSIZ*8];
 #endif
 
@@ -1107,7 +1098,7 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 
 #if 0 /* This is broken - the lseek() below broke it, but it fixed a lot
 	 of other problems..  Thus possible 'cache' goes away.. */
-#ifndef	USE_MMAP
+#ifndef	HAVE_MMAP
 	/* can we use cache of message body data */
 	if (convertmode == _CONVERT_NONE &&
 	    readalready != 0) {
@@ -1127,7 +1118,7 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 	  if (fflush(fp) != 0)
 	    return EX_IOERR;
 	}
-#endif /* !USE_MMAP */
+#endif /* !HAVE_MMAP */
 #endif /* 0 */
 
 	if (lseek(dp->msgfd, dp->msgbodyoffset, SEEK_SET) < 0L)
@@ -1136,10 +1127,10 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 	bufferfull = 0;
 	lastwasnl = 1;	/* we are guaranteed to have a \n after the header */
 	if (convertmode == _CONVERT_NONE) {
-#ifdef	USE_MMAP
+#ifdef	HAVE_MMAP
 	  char *let_buffer = dp->let_buffer + dp->msgbodyoffset;
 	  i = dp->let_end - dp->let_buffer - dp->msgbodyoffset;
-#else /* !USE_MMAP */
+#else /* !HAVE_MMAP */
 	  while (1) {
 	    if ((i = read(dp->msgfd, let_buffer, sizeof(let_buffer))) == 0)
 	      break;
@@ -1147,7 +1138,7 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 	      strcpy(remotemsg, "smtp; 500 (Read error from message file!?)");
 	      return EX_IOERR;
 	    }
-#endif /* !USE_MMAP */
+#endif /* !HAVE_MMAP */
 	    lastwasnl = (let_buffer[i-1] == '\n');
 	    rc = writebuf(fp, let_buffer, i);
 	    hsize += rc;
@@ -1165,7 +1156,7 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 		      strerror(errno), hsize, msize, (hsize*100+msize/2)/msize);
 	      return EX_IOERR;
 	    }
-#ifndef USE_MMAP
+#ifndef HAVE_MMAP
 	    bufferfull++;
 	    readalready = i;
 	  }
@@ -1174,13 +1165,13 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 	  /* Various esoteric conversion modes..
 	     We are better to feed writemimeline() with LINES
 	     instead of blocks of data.. */
-#ifndef	USE_MMAP /* Classical way to read in things */
+#ifndef	HAVE_MMAP /* Classical way to read in things */
 	  char iobuf[BUFSIZ];
 	  FILE *mfp = fdopen(dp->msgfd,"r");
 	  setvbuf(mfp, iobuf, _IOFBF, sizeof(iobuf));
 #define MFPCLOSE i = dup(dp->msgfd); fclose(mfp); dup2(i,dp->msgfd); close(i);
 	  bufferfull = 0;
-#else /* USE_MMAP */
+#else /* HAVE_MMAP */
 	  char *s = dp->let_buffer + dp->msgbodyoffset;
 
 #define MFPCLOSE
@@ -1190,18 +1181,18 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 
 	  /* we are assuming to be positioned properly
 	     at the start of the message body */
-#ifndef	USE_MMAP
+#ifndef	HAVE_MMAP
 	  readalready = 0;
 #endif
 	  lastwasnl = 0;
 	  for (;;) {
-#ifndef USE_MMAP
+#ifndef HAVE_MMAP
 	    if ((i = cfgets(let_buffer, sizeof(let_buffer), mfp)) == EOF)
 	      break;
 	    /* It MAY be malformed -- if it has a BUFSIZ*8 length
 	       line in it, IT CAN'T BE MIME  :-/		*/
 	    lastwasnl = (let_buffer[i-1] == '\n');
-#else /* USE_MMAP */
+#else /* HAVE_MMAP */
 	    char *let_buffer = s, *s2 = s;
 	    if (s >= dp->let_end) break; /* EOF */
 	    i = 0;
@@ -1226,11 +1217,11 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 	      MFPCLOSE
 	      return EX_IOERR;
 	    }
-#ifdef USE_MMAP
+#ifdef HAVE_MMAP
 	    s = s2; /* Advance one linefull.. */
 #endif
 	  }
-#ifndef	USE_MMAP
+#ifndef	HAVE_MMAP
 	  if (i == EOF && !feof(mfp)) {
 	    strcpy(remotemsg, "500 (Read error from message file!?)");
 	    MFPCLOSE
@@ -1251,13 +1242,13 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 	    sprintf(remotemsg,
 		    "500 (let_buffer write IO-error! [%s] DATA %d/%d [%d%%])",
 		    strerror(errno), hsize, msize, (hsize*100+msize/2)/msize);
-#ifndef	USE_MMAP
+#ifndef	HAVE_MMAP
 	    if (bufferfull > 1) readalready = 0;
 #endif
 	    return EX_IOERR;
 	  }
 
-#ifndef USE_MMAP
+#ifndef HAVE_MMAP
 	if (bufferfull > 1)	/* not all in memory, need to reread */
 	  readalready = 0;
 #endif
@@ -2055,7 +2046,7 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	return EX_UNAVAILABLE;
 }
 
-SIGNAL_TYPE
+RETSIGTYPE
 sig_pipe(sig)
 int sig;
 {
@@ -2065,7 +2056,7 @@ int sig;
 	}
 }
 
-SIGNAL_TYPE
+RETSIGTYPE
 sig_alarm(sig)
 int sig;
 {
@@ -2127,7 +2118,12 @@ smtp_flush()
 	pipeindex = 0;
 }
 
-#ifdef	USE_SELECT
+#ifdef	HAVE_SELECT
+
+#ifdef _AIX /* Defines NFDBITS, et.al. */
+#include <sys/types.h>
+#include <sys/select.h>
+#endif
 
 
 #if	defined(BSD4_3) || defined(sun)
@@ -2154,11 +2150,6 @@ typedef long	fd_mask;
 typedef	struct fd_set { fd_mask	fds_bits[1]; } fd_set;
 #endif	/* notdef */
 
-#ifdef _AIX
-#include <sys/types.h>
-#include <sys/select.h>
-#endif
-
 #ifndef	FD_SET
 #define	FD_SET(n, p)	((p)->fds_bits[0] |= (1 << (n)))
 #define	FD_CLR(n, p)	((p)->fds_bits[0] &= ~(1 << (n)))
@@ -2167,35 +2158,24 @@ typedef	struct fd_set { fd_mask	fds_bits[1]; } fd_set;
 #endif	/* !FD_SET */
 #endif	/* !NFDBITS */
 
-#if defined(_AIX) && !defined(FD_SET) /* Pre AIX 4.1 ? */
-#define _FD_SET(sock,var) FD_SET(sock,&var.fdsmask)
-#define _FD_CLR(sock,var) FD_CLR(sock,&var.fdsmask)
-#define _FD_ZERO(var) FD_ZERO(&var.fdsmask)
-#define _FD_ISSET(i,var) FD_ISSET(i,&var.fdsmask)
-#else
 #ifdef FD_SET
 #define _FD_SET(sock,var) FD_SET(sock,&var)
 #define _FD_CLR(sock,var) FD_CLR(sock,&var)
-#define _FD_ZERO(var) FD_ZERO(&var)
-#define _FD_ISSET(i,var) FD_ISSET(i,&var)
+#define _FD_ZERO(var)     FD_ZERO(&var)
+#define _FD_ISSET(i,var)  FD_ISSET(i,&var)
 #else
 #define _FD_SET(sock,var) var |= (1 << sock)
 #define _FD_CLR(sock,var) var &= ~(1 << sock)
-#define _FD_ZERO(var) var = 0
+#define _FD_ZERO(var)     var = 0
 #define _FD_ISSET(i,var) ((var & (1 << i)) != 0)
 #endif
-#endif
 
 int select_sleep(fd,timeout)
 int fd, timeout;
 {
 	struct timeval tv;
 	int rc;
-#ifdef _AIX
-	struct { fd_set fdsmask; fd_set msgsmask; } rdmask;
-#else
 	fd_set rdmask;
-#endif
 
 	tv.tv_sec = timeout;
 	tv.tv_usec = 0;
@@ -2209,7 +2189,7 @@ int fd, timeout;
 	  return 0;
 	return 1;    /* interrupt, or some such.. */
 }
-#else /* not USE_SELECT */
+#else /* not HAVE_SELECT */
 int select_sleep(fd, timeout)
 int fd, timeout;
 {
@@ -2823,18 +2803,21 @@ report(va_alist)
 	char *cp, buf[8192];
 	
 	va_start(ap);
-	sprintf(buf, "-%s ", remotehost);
+	if (smtp_isopen)
+	  sprintf(buf, "-%s ", remotehost);
+	else
+	  sprintf(buf, "-[%s] ", remotehost);
 #ifdef	notdef
 	if (logfp)
 	  sprintf(buf+strlen(buf), ">>%s ", logfile);
 	strcat(buf, "# ");
 #endif
 	cp = va_arg(ap, char *);
-#ifdef	USE_VFPRINTF
+#ifdef	HAVE_VPRINTF
 	vsprintf(buf+strlen(buf), cp, ap);
-#else	/* !USE_VFPRINTF */
+#else	/* !HAVE_VPRINTF */
 	sprintf(buf+strlen(buf), cp, va_arg(ap, char *));
-#endif	/* USE_VFPRINTF */
+#endif	/* HAVE_VPRINTF */
 	for (cp = buf+strlen(buf); cp < buf + (eocmdline - cmdline); ++cp)
 	  *cp = ' ';
 	buf[eocmdline-cmdline] = '\0';
@@ -3242,7 +3225,7 @@ int
 check_7bit_cleanness(dp)
 struct ctldesc *dp;
 {
-#ifdef	USE_MMAP
+#ifdef	HAVE_MMAP
 	/* With MMAP()ed spool file it is sweet and simple.. */
 	register char *s = dp->let_buffer + dp->msgbodyoffset;
 	while (s < dp->let_end)
@@ -3251,7 +3234,7 @@ struct ctldesc *dp;
 	  else
 	    ++s;
 	return 1;
-#else /* !USE_MMAP */
+#else /* !HAVE_MMAP */
 
 	register int i;
 	register int bufferfull;
@@ -3294,7 +3277,7 @@ struct ctldesc *dp;
 
 	/* Set indication! */
 	return 1;
-#endif /* !USE_MMAP */
+#endif /* !HAVE_MMAP */
 }
 
 void
